@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2016-2021 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2016-2022 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -26,6 +26,7 @@
 
 #include "config.h"
 #include "asc-canvas.h"
+#include "asc-canvas-private.h"
 
 #include <cairo.h>
 #include <cairo-ft.h>
@@ -153,9 +154,13 @@ asc_canvas_render_svg (AscCanvas *canvas, GInputStream *stream, GError **error)
 {
 	AscCanvasPrivate *priv = GET_PRIVATE (canvas);
 	RsvgHandle *handle = NULL;
-	RsvgDimensionData dims;
 	gboolean ret = FALSE;
-	double w, h;
+	gdouble srf_width, srf_height;
+#if LIBRSVG_CHECK_VERSION(2, 52, 0)
+	RsvgRectangle viewport;
+#else
+	RsvgDimensionData dims;
+#endif
 
 	/* NOTE: unfortunately, Cairo/RSvg may use Fontconfig internally, so
 	 * we need to lock this down since a parallel-processed font
@@ -169,16 +174,38 @@ asc_canvas_render_svg (AscCanvas *canvas, GInputStream *stream, GError **error)
 						   error);
 	if (handle == NULL)
 		goto out;
+	rsvg_handle_set_dpi (handle, 100);
 
-	rsvg_handle_get_dimensions (handle, &dims);
-	w = (double) cairo_image_surface_get_width (priv->srf);
-	h = (double) cairo_image_surface_get_height (priv->srf);
-
-	/* cairo_translate (cr, (w - dims.width) / 2, (h - dims.height) / 2); */
-	cairo_scale (priv->cr, w / dims.width, h / dims.height);
+	srf_width = (gdouble) cairo_image_surface_get_width (priv->srf);
+	srf_height = (gdouble) cairo_image_surface_get_height (priv->srf);
 
 	cairo_save (priv->cr);
-        if (!rsvg_handle_render_cairo (handle, priv->cr)) {
+
+#if LIBRSVG_CHECK_VERSION(2, 52, 0)
+	viewport.x = 0;
+	viewport.y = 0;
+	viewport.width = srf_width;
+	viewport.height = srf_height;
+
+	ret = rsvg_handle_render_document (handle,
+					   priv->cr,
+					   &viewport,
+					   error);
+	if (!ret) {
+		cairo_restore (priv->cr);
+		g_prefix_error (error, "SVG graphic rendering failed:");
+		goto out;
+	}
+#else
+	rsvg_handle_get_dimensions (handle, &dims);
+
+	/* cairo_translate (cr, (srf_width - dims.width) / 2, (srf_height - dims.height) / 2); */
+	cairo_scale (priv->cr,
+		     srf_width / dims.width,
+		     srf_height / dims.height);
+
+	ret = rsvg_handle_render_cairo (handle, priv->cr);
+	if (!ret) {
 		cairo_restore (priv->cr);
 		g_set_error_literal (error,
 				     ASC_CANVAS_ERROR,
@@ -186,8 +213,9 @@ asc_canvas_render_svg (AscCanvas *canvas, GInputStream *stream, GError **error)
 				     "SVG graphic rendering failed.");
 		goto out;
 	}
-	ret = TRUE;
+#endif
 
+	ret = TRUE;
 out:
 	if (handle != NULL)
 		g_object_unref (handle);
@@ -389,7 +417,8 @@ asc_canvas_save_png (AscCanvas *canvas, const gchar *fname, GError **error)
 		g_set_error (error,
 			     ASC_CANVAS_ERROR,
 			     ASC_CANVAS_ERROR_FONT,
-			     "Could not save canvas to PNG: %i", status);
+			     "Could not save canvas to PNG: %s",
+			     cairo_status_to_string (status));
 		return FALSE;
 	}
 
