@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2016-2021 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2016-2022 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -62,18 +62,6 @@ gboolean	asc_canvas_render_svg (AscCanvas* canvas,
 					GInputStream *stream,
 					GError** error);
 
-gboolean	asc_canvas_draw_text_line (AscCanvas *canvas,
-					   AscFont *font,
-					   const gchar *text,
-					   gint border_width,
-					   GError **error);
-gboolean	asc_canvas_draw_text (AscCanvas *canvas,
-					AscFont *font,
-					const gchar *text,
-					gint border_width,
-					gint line_pad,
-					GError **error);
-
 gboolean	asc_canvas_save_png (AscCanvas *canvas,
 					const gchar *fname,
 					GError **error);
