@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2016-2021 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2016-2022 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -43,6 +43,7 @@ typedef struct
 {
 	gboolean	use_optipng;
 	gchar		*optipng_bin;
+	gchar		*ffprobe_bin;
 	gchar		*tmp_dir;
 
 	GMutex		pangrams_mutex;
@@ -56,6 +57,7 @@ G_DEFINE_TYPE_WITH_PRIVATE (AscGlobals, asc_globals, G_TYPE_OBJECT)
 #define GET_PRIVATE(o) (asc_globals_get_instance_private (o))
 
 static AscGlobals *g_globals = NULL;
+static GMutex g_globals_mutex;
 
 /**
  * asc_compose_error_quark:
@@ -74,8 +76,9 @@ asc_compose_error_quark (void)
 static GObject*
 asc_globals_constructor (GType type, guint n_construct_properties, GObjectConstructParam *construct_properties)
 {
-	if (g_globals)
-		return g_object_ref (G_OBJECT (g_globals));
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&g_globals_mutex);
+	if (g_globals != NULL)
+		return G_OBJECT (g_globals);
 	else
 		return G_OBJECT_CLASS (asc_globals_parent_class)->constructor (type, n_construct_properties, construct_properties);
 }
@@ -88,6 +91,7 @@ asc_globals_finalize (GObject *object)
 
 	g_free (priv->tmp_dir);
 	g_free (priv->optipng_bin);
+	g_free (priv->ffprobe_bin);
 
 	g_mutex_clear (&priv->pangrams_mutex);
 	if (priv->pangrams_en != NULL)
@@ -104,15 +108,22 @@ static void
 asc_globals_init (AscGlobals *globals)
 {
 	AscGlobalsPrivate *priv = GET_PRIVATE (globals);
+	g_autofree gchar *tmp_str1 = NULL;
+	g_autofree gchar *tmp_str2 = NULL;
 	g_assert (g_globals == NULL);
 	g_globals = globals;
 
-	priv->tmp_dir = g_build_filename (g_get_tmp_dir (), "as-compose", NULL);
+	tmp_str1 = as_random_alnum_string (6);
+	tmp_str2 = g_strconcat ("as-compose_", tmp_str1, NULL);
+	priv->tmp_dir = g_build_filename (g_get_tmp_dir (), tmp_str2, NULL);
 
 	priv->optipng_bin = g_find_program_in_path ("optipng");
 	if (priv->optipng_bin != NULL)
 		priv->use_optipng = TRUE;
 
+	priv->ffprobe_bin = g_find_program_in_path ("ffprobe");
+
+	g_mutex_init (&priv->hint_tags_mutex);
 	g_mutex_init (&priv->pangrams_mutex);
 }
 
@@ -130,6 +141,22 @@ asc_globals_get_priv (void)
 	return GET_PRIVATE (g_object_new (ASC_TYPE_GLOBALS, NULL));
 }
 
+/**
+ * asc_globals_clear:
+ *
+ * Clear all global state and restore defaults.
+ */
+void
+asc_globals_clear (void)
+{
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&g_globals_mutex);
+	if (g_globals == NULL)
+		return;
+
+	g_object_unref (g_globals);
+	g_globals = NULL;
+}
+
 /**
  * asc_globals_get_tmp_dir:
  *
@@ -225,6 +252,31 @@ asc_globals_set_optipng_binary (const gchar *path)
 		priv->use_optipng = FALSE;
 }
 
+/**
+ * asc_globals_get_ffprobe_binary:
+ *
+ * Get path to the "ffprobe" binary we should use.
+ **/
+const gchar*
+asc_globals_get_ffprobe_binary (void)
+{
+	AscGlobalsPrivate *priv = asc_globals_get_priv ();
+	return priv->ffprobe_bin;
+}
+
+/**
+ * asc_globals_set_ffprobe_binary:
+ *
+ * Set path to the "ffprobe" binary we should use.
+ **/
+void
+asc_globals_set_ffprobe_binary (const gchar *path)
+{
+	AscGlobalsPrivate *priv = asc_globals_get_priv ();
+	g_free (priv->ffprobe_bin);
+	priv->ffprobe_bin = g_strdup (path);
+}
+
 /**
  * asc_globals_get_pangrams_for:
  *
@@ -328,24 +380,45 @@ asc_globals_create_hint_tag_table ()
 
 /**
  * asc_globals_add_hint_tag:
+ * @tag: the tag-ID to add
+ * @severity: the tag severity as #AsIssueSeverity
+ * @explanation: the tag explanatory message
+ * @overrideExisting: whether an existing tag should be replaced
  *
  * Register a new hint tag. If a previous tag with the given name
- * already existed, the existing tag will not be replaced.
+ * already existed, the existing tag will not be replaced unless
+ * @overrideExisting is set to %TRUE.
+ * Please be careful when overriding tags! Tag severities can not
+ * be lowered by overriding a tag.
  *
  * Returns: %TRUE if the tag was registered and did not exist previously.
  */
 gboolean
-asc_globals_add_hint_tag (const gchar *tag, AsIssueSeverity severity, const gchar *explanation)
+asc_globals_add_hint_tag (const gchar *tag,
+			  AsIssueSeverity severity,
+			  const gchar *explanation,
+			  gboolean overrideExisting)
 {
 	AscGlobalsPrivate *priv = asc_globals_get_priv ();
 	AscHintTag *htag;
+	AscHintTag *e_htag;
 	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&priv->hint_tags_mutex);
 	g_return_val_if_fail (tag != NULL, FALSE);
 
 	if (priv->hint_tags == NULL)
 		asc_globals_create_hint_tag_table ();
-	if (g_hash_table_contains (priv->hint_tags, tag))
-		return FALSE;
+
+	e_htag = g_hash_table_lookup (priv->hint_tags, tag);
+	if (e_htag != NULL) {
+		if (overrideExisting) {
+			/* make sure we don't permit lowering severities */
+			if (severity > e_htag->severity)
+				severity = e_htag->severity;
+		} else {
+			/* don't allow the override */
+			return FALSE;
+		}
+	}
 
 	htag = asc_hint_tag_new (tag, severity, explanation);
 	g_hash_table_insert (priv->hint_tags,
