@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2016-2021 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2016-2022 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -43,6 +43,8 @@ typedef enum {
 #define	ASC_COMPOSE_ERROR	asc_compose_error_quark ()
 GQuark				asc_compose_error_quark (void);
 
+void				asc_globals_clear (void);
+
 const gchar			*asc_globals_get_tmp_dir (void);
 const gchar			*asc_globals_get_tmp_dir_create (void);
 void				asc_globals_set_tmp_dir (const gchar *path);
@@ -53,9 +55,13 @@ void				asc_globals_set_use_optipng (gboolean enabled);
 const gchar			*asc_globals_get_optipng_binary (void);
 void				asc_globals_set_optipng_binary (const gchar *path);
 
+const gchar			*asc_globals_get_ffprobe_binary (void);
+void				asc_globals_set_ffprobe_binary (const gchar *path);
+
 gboolean			asc_globals_add_hint_tag (const gchar *tag,
 							  AsIssueSeverity severity,
-							  const gchar *explanation);
+							  const gchar *explanation,
+							  gboolean overrideExisting);
 gchar				**asc_globals_get_hint_tags ();
 AsIssueSeverity			asc_globals_hint_tag_severity (const gchar *tag);
 const gchar			*asc_globals_hint_tag_explanation (const gchar *tag);
