@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
  * Copyright (C) 2018 Richard Hughes <richard@hughsie.com>
- * Copyright (C) 2018-2021 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2018-2022 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -350,7 +350,7 @@ as_agreement_section_to_xml_node (AsAgreementSection *agreement_section, AsConte
 	xmlNewProp (asnode, (xmlChar*) "type", (xmlChar*) priv->kind);
 
 	as_xml_add_localized_text_node (asnode, "name", priv->name);
-	as_xml_add_description_node (ctx, asnode, priv->description);
+	as_xml_add_description_node (ctx, asnode, priv->description, TRUE);
 }
 
 /**
