@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2012-2021 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2012-2022 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -67,10 +67,9 @@ const gchar		*as_component_get_architecture (AsComponent *cpt);
 void			 as_component_set_architecture (AsComponent *cpt,
 							const gchar *arch);
 
-void			 as_component_create_token_cache (AsComponent *cpt);
-GHashTable		*as_component_get_token_cache_table (AsComponent *cpt);
-void			 as_component_set_token_cache_valid (AsComponent *cpt,
-							     gboolean valid);
+void			as_component_create_token_cache (AsComponent *cpt);
+GPtrArray		*as_component_generate_tokens_for (AsComponent *cpt,
+							   AsSearchTokenMatch token_kind);
 
 void			as_component_set_ignored (AsComponent *cpt,
 						  gboolean ignore);
