@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2012-2021 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2012-2022 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -42,6 +42,7 @@
 #include "as-relation.h"
 #include "as-agreement.h"
 #include "as-review.h"
+#include "as-branding.h"
 
 G_BEGIN_DECLS
 
@@ -218,6 +219,11 @@ AsComponentKind		as_component_get_kind (AsComponent *cpt);
 void			as_component_set_kind (AsComponent *cpt,
 						AsComponentKind value);
 
+const gchar		*as_component_get_date_eol (AsComponent *cpt);
+void			as_component_set_date_eol (AsComponent *cpt,
+						   const gchar *date);
+guint64			as_component_get_timestamp_eol (AsComponent *cpt);
+
 const gchar		*as_component_get_origin (AsComponent *cpt);
 void			as_component_set_origin (AsComponent *cpt,
 							const gchar *origin);
@@ -234,6 +240,8 @@ gchar			*as_component_get_pkgname (AsComponent *cpt);
 gchar			**as_component_get_pkgnames (AsComponent *cpt);
 void			as_component_set_pkgnames (AsComponent *cpt,
 							gchar **packages);
+void			as_component_set_pkgname (AsComponent *cpt,
+						  const gchar *pkgname);
 
 const gchar		*as_component_get_source_pkgname (AsComponent *cpt);
 void			as_component_set_source_pkgname (AsComponent *cpt,
@@ -338,6 +346,7 @@ gint			as_component_get_language (AsComponent *cpt,
 void			as_component_add_language (AsComponent *cpt,
 							const gchar *locale,
 							gint percentage);
+void			as_component_clear_languages (AsComponent *cpt);
 
 GPtrArray		*as_component_get_translations (AsComponent *cpt);
 void			as_component_add_translation (AsComponent *cpt,
@@ -389,16 +398,33 @@ AsContentRating		*as_component_get_content_rating (AsComponent *cpt,
 void			as_component_add_content_rating (AsComponent *cpt,
 							 AsContentRating *content_rating);
 
-GPtrArray		*as_component_get_recommends (AsComponent *cpt);
 GPtrArray		*as_component_get_requires (AsComponent *cpt);
+GPtrArray		*as_component_get_recommends (AsComponent *cpt);
+GPtrArray		*as_component_get_supports (AsComponent *cpt);
 void			as_component_add_relation (AsComponent *cpt,
 						   AsRelation *relation);
 
+GPtrArray		*as_component_get_agreements (AsComponent *cpt);
 void			as_component_add_agreement (AsComponent *cpt,
 						    AsAgreement *agreement);
 AsAgreement		*as_component_get_agreement_by_kind (AsComponent *cpt,
 							     AsAgreementKind kind);
 
+AsBranding		*as_component_get_branding (AsComponent *cpt);
+void			as_component_set_branding (AsComponent *cpt,
+						    AsBranding *branding);
+
+void			as_component_clear_tags (AsComponent *cpt);
+gboolean		as_component_add_tag (AsComponent *cpt,
+					      const gchar *ns,
+					      const gchar *tag);
+gboolean		as_component_remove_tag (AsComponent *cpt,
+						 const gchar *ns,
+						 const gchar *tag);
+gboolean		as_component_has_tag (AsComponent *cpt,
+					      const gchar *ns,
+					      const gchar *tag);
+
 const gchar		*as_component_get_name_variant_suffix (AsComponent *cpt);
 void			as_component_set_name_variant_suffix (AsComponent *cpt,
 								const gchar *value,
