@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2012-2021 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2012-2022 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -101,6 +101,7 @@ AS_INTERNAL_VISIBLE
 gboolean		as_is_empty (const gchar *str);
 GDateTime		*as_iso8601_to_datetime (const gchar *iso_date);
 
+AS_INTERNAL_VISIBLE
 gboolean		as_utils_delete_dir_recursive (const gchar* dirname);
 
 AS_INTERNAL_VISIBLE
@@ -113,6 +114,7 @@ GPtrArray		*as_utils_find_files (const gchar *dir,
 						gboolean recursive,
 						GError **error);
 
+AS_INTERNAL_VISIBLE
 gboolean		as_utils_is_root (void);
 
 AS_INTERNAL_VISIBLE
@@ -131,6 +133,8 @@ void			as_hash_table_string_keys_to_array (GHashTable *table,
 							    GPtrArray *array);
 
 gboolean		as_touch_location (const gchar *fname);
+
+AS_INTERNAL_VISIBLE
 void			as_reset_umask (void);
 
 AS_INTERNAL_VISIBLE
@@ -168,8 +172,6 @@ gchar			*as_ptr_array_to_str (GPtrArray *array,
 AS_INTERNAL_VISIBLE
 gchar			*as_filebasename_from_uri (const gchar *uri);
 
-gchar			*as_date_time_format_iso8601 (GDateTime *datetime);
-
 AS_INTERNAL_VISIBLE
 gchar			*as_strstripnl (gchar *string);
 
@@ -190,10 +192,11 @@ gboolean		as_utils_is_platform_triplet_arch (const gchar *arch);
 gboolean		as_utils_is_platform_triplet_oskernel (const gchar *os);
 gboolean		as_utils_is_platform_triplet_osenv (const gchar *env);
 
-gchar			*as_get_user_cache_dir ();
+gchar			*as_get_user_cache_dir (GError **error);
 
 gboolean		as_unichar_accepted (gunichar c);
 
+AS_INTERNAL_VISIBLE
 gchar			*as_sanitize_text_spaces (const gchar *text);
 
 AS_INTERNAL_VISIBLE
@@ -204,6 +207,8 @@ gchar			*as_utils_find_stock_icon_filename_full (const gchar *root_dir,
 								 guint icon_size,
 								 guint icon_scale,
 								 GError **error);
+AS_INTERNAL_VISIBLE
+void			as_utils_ensure_resources ();
 
 #pragma GCC visibility pop
 G_END_DECLS
