@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2012-2021 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2012-2022 Matthias Klumpp <matthias@tenstral.net>
  * Copyright (C) 2014-2016 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
@@ -118,7 +118,10 @@ as_markup_strsplit_words (const gchar *text, guint line_len)
 		if (curline_char_count + token_unilen < line_len) {
 			/* we can't just check for a suffix \n here, as tokens may contain internal linebreaks */
 			if (token_has_linebreak) {
-				g_string_append (curline, (tokens[i][0] == '\0')? " " : tokens[i]);
+				if (tokens[i][0] == '\0')
+					g_string_append_c (curline, ' ');
+				else
+					g_string_append_printf (curline, "%s ", tokens[i]);
 				g_ptr_array_add (lines, g_strdup (curline->str));
 				g_string_truncate (curline, 0);
 				curline_char_count = 0;
@@ -187,7 +190,7 @@ as_sanitize_text_spaces (const gchar *text)
 	strv = g_strsplit (text, "\n", -1);
 	for (guint i = 0; strv[i] != NULL; ++i)
 		g_strstrip (strv[i]);
-	return g_strjoinv (" ", strv);
+	return g_strstrip (g_strjoinv (" ", strv));
 }
 
 /**
@@ -250,6 +253,11 @@ as_description_markup_convert (const gchar *markup, AsMarkupKind to_kind, GError
 			g_autofree gchar *clean_text = NULL;
 			g_autofree gchar *text_content = (gchar*) xmlNodeGetContent (iter);
 
+			/* Apparently the element is empty, which is odd. But we better add it instead
+			 * of completely ignoring it. */
+			if (text_content == NULL)
+				text_content = g_strdup ("");
+
 			/* remove extra whitespaces and linebreaks */
 			clean_text = as_sanitize_text_spaces (text_content);
 
@@ -286,6 +294,11 @@ as_description_markup_convert (const gchar *markup, AsMarkupKind to_kind, GError
 					g_autofree gchar *item_content = (gchar*) xmlNodeGetContent (iter2);
 					entry_no++;
 
+					/* Apparently the item text is empty, which is odd.
+					 * Let's add an empty entry, instead of ignoring it entirely. */
+					if (item_content == NULL)
+						item_content = g_strdup ("");
+
 					/* remove extra whitespaces and linebreaks */
 					clean_item = as_sanitize_text_spaces (item_content);
 
@@ -406,7 +419,7 @@ as_utils_delete_dir_recursive (const gchar* dirname)
 	gboolean ret = FALSE;
 	GFile *dir;
 	GFileEnumerator *enr;
-	GFileInfo *info;
+	g_autoptr(GFileInfo) info = NULL;
 	g_return_val_if_fail (dirname != NULL, FALSE);
 
 	if (!g_file_test (dirname, G_FILE_TEST_IS_DIR))
@@ -423,21 +436,18 @@ as_utils_delete_dir_recursive (const gchar* dirname)
 	if (error != NULL)
 		goto out;
 	while (info != NULL) {
-		gchar *path;
-		path = g_build_filename (dirname, g_file_info_get_name (info), NULL);
-		if (g_file_test (path, G_FILE_TEST_IS_DIR)) {
+		g_autofree gchar *path = g_build_filename (dirname, g_file_info_get_name (info), NULL);
+		if (g_file_test (path, G_FILE_TEST_IS_DIR))
 			as_utils_delete_dir_recursive (path);
-		} else {
+		else
 			g_remove (path);
-		}
 		g_object_unref (info);
 		info = g_file_enumerator_next_file (enr, NULL, &error);
 		if (error != NULL)
 			goto out;
 	}
-	if (g_file_test (dirname, G_FILE_TEST_EXISTS)) {
+	if (g_file_test (dirname, G_FILE_TEST_EXISTS))
 		g_rmdir (dirname);
-	}
 	ret = TRUE;
 
 out:
@@ -595,8 +605,7 @@ gchar*
 as_get_current_locale (void)
 {
 	const gchar * const *locale_names;
-	gchar *tmp;
-	gchar *locale = NULL;
+	const gchar *locale = NULL;
 
 	/* use LANGUAGE, LC_ALL, LC_MESSAGES and LANG */
 	locale_names = g_get_language_names ();
@@ -610,17 +619,13 @@ as_get_current_locale (void)
 		 * multiple caches on systems which generate them via a backend in PackageKit. */
 		const gchar *env_lang = g_getenv ("LANG");
 		if ((env_lang != NULL) && (g_strstr_len (env_lang, -1, "_") != NULL))
-			locale = g_strdup (env_lang);
+			locale = env_lang;
 	}
 	if (locale == NULL)
-		locale = g_strdup (locale_names[0]);
+		locale = locale_names[0];
 
-	/* set active locale without UTF-8 suffix, UTF-8 is default in AppStream */
-	tmp = g_strstr_len (locale, -1, ".UTF-8");
-	if (tmp != NULL)
-		*tmp = '\0';
-
-	return locale;
+	/* return active locale without UTF-8 suffix, UTF-8 is default in AppStream */
+	return as_locale_strip_encoding (locale);
 }
 
 /**
@@ -865,7 +870,7 @@ as_get_current_arch (void)
 	if (g_strcmp0 (uts.machine, "x86_64") == 0) {
 		arch = g_strdup ("amd64");
 	} else if (g_pattern_match_simple ("i?86", uts.machine)) {
-		arch = g_strdup ("ia32");
+		arch = g_strdup ("i386");
 	} else if (g_strcmp0 (uts.machine, "aarch64")) {
 		arch = g_strdup ("arm64");
 	} else {
@@ -915,6 +920,12 @@ as_utils_locale_to_language (const gchar *locale)
 	tmp = g_strstr_len (country_code, -1, "_");
 	if (tmp != NULL)
 		*tmp = '\0';
+
+	/* return the part before any "@" for locale with modifiers like "ca@valencia" */
+	tmp = g_strstr_len (country_code, -1, "@");
+	if (tmp != NULL)
+		*tmp = '\0';
+
 	return country_code;
 }
 
@@ -1034,6 +1045,24 @@ as_utils_search_token_valid (const gchar *token)
 	return TRUE;
 }
 
+/**
+ * as_utils_ensure_resources:
+ *
+ * Perform a sanity check to ensure GResource can be loaded.
+ */
+void
+as_utils_ensure_resources ()
+{
+	static GMutex mutex;
+	GResource *resource = NULL;
+
+	g_mutex_lock (&mutex);
+	resource = as_get_resource ();
+	if (resource == NULL)
+		g_error ("Failed to load internal resources: as_get_resource() returned NULL!");
+	g_mutex_unlock (&mutex);
+}
+
 /**
  * as_utils_is_category_id:
  * @category_name: a XDG category name, e.g. "ProjectManagement"
@@ -1051,13 +1080,15 @@ as_utils_is_category_name (const gchar *category_name)
 {
 	g_autoptr(GBytes) data = NULL;
 	g_autofree gchar *key = NULL;
+	GResource *resource = as_get_resource ();
+	g_assert (resource != NULL);
 
 	/* custom spec-extensions are generally valid if prefixed correctly */
 	if (g_str_has_prefix (category_name, "X-"))
 		return TRUE;
 
 	/* load the readonly data section and look for the category name */
-	data = g_resource_lookup_data (as_get_resource (),
+	data = g_resource_lookup_data (resource,
 				       "/org/freedesktop/appstream/xdg-category-names.txt",
 				       G_RESOURCE_LOOKUP_FLAGS_NONE,
 				       NULL);
@@ -1083,9 +1114,11 @@ as_utils_is_tld (const gchar *tld)
 {
 	g_autoptr(GBytes) data = NULL;
 	g_autofree gchar *key = NULL;
+	GResource *resource = as_get_resource ();
+	g_assert (resource != NULL);
 
 	/* load the readonly data section and look for the TLD */
-	data = g_resource_lookup_data (as_get_resource (),
+	data = g_resource_lookup_data (resource,
 				       "/org/freedesktop/appstream/iana-filtered-tld-list.txt",
 				       G_RESOURCE_LOOKUP_FLAGS_NONE,
 				       NULL);
@@ -1111,9 +1144,11 @@ as_utils_is_desktop_environment (const gchar *desktop)
 {
 	g_autoptr(GBytes) data = NULL;
 	g_autofree gchar *key = NULL;
+	GResource *resource = as_get_resource ();
+	g_assert (resource != NULL);
 
 	/* load the readonly data section and look for the desktop environment name */
-	data = g_resource_lookup_data (as_get_resource (),
+	data = g_resource_lookup_data (resource,
 				       "/org/freedesktop/appstream/desktop-environments.txt",
 				       G_RESOURCE_LOOKUP_FLAGS_NONE,
 				       NULL);
@@ -1139,6 +1174,7 @@ as_utils_is_platform_triplet_arch (const gchar *arch)
 {
 	g_autoptr(GBytes) data = NULL;
 	g_autofree gchar *key = NULL;
+	GResource *resource = NULL;
 
 	if (arch == NULL)
 		return FALSE;
@@ -1147,8 +1183,11 @@ as_utils_is_platform_triplet_arch (const gchar *arch)
 	if (g_strcmp0 (arch, "any") == 0)
 		return TRUE;
 
+	resource = as_get_resource ();
+	g_assert (resource != NULL);
+
 	/* load the readonly data section */
-	data = g_resource_lookup_data (as_get_resource (),
+	data = g_resource_lookup_data (resource,
 				       "/org/freedesktop/appstream/platform_arch.txt",
 				       G_RESOURCE_LOOKUP_FLAGS_NONE,
 				       NULL);
@@ -1174,6 +1213,7 @@ as_utils_is_platform_triplet_oskernel (const gchar *os)
 {
 	g_autoptr(GBytes) data = NULL;
 	g_autofree gchar *key = NULL;
+	GResource *resource;
 
 	if (os == NULL)
 		return FALSE;
@@ -1182,8 +1222,11 @@ as_utils_is_platform_triplet_oskernel (const gchar *os)
 	if (g_strcmp0 (os, "any") == 0)
 		return TRUE;
 
+	resource = as_get_resource ();
+	g_assert (resource != NULL);
+
 	/* load the readonly data section */
-	data = g_resource_lookup_data (as_get_resource (),
+	data = g_resource_lookup_data (resource,
 				       "/org/freedesktop/appstream/platform_os.txt",
 				       G_RESOURCE_LOOKUP_FLAGS_NONE,
 				       NULL);
@@ -1209,6 +1252,7 @@ as_utils_is_platform_triplet_osenv (const gchar *env)
 {
 	g_autoptr(GBytes) data = NULL;
 	g_autofree gchar *key = NULL;
+	GResource *resource;
 
 	if (env == NULL)
 		return FALSE;
@@ -1217,8 +1261,11 @@ as_utils_is_platform_triplet_osenv (const gchar *env)
 	if (g_strcmp0 (env, "any") == 0)
 		return TRUE;
 
+	resource = as_get_resource ();
+	g_assert (resource != NULL);
+
 	/* load the readonly data section */
-	data = g_resource_lookup_data (as_get_resource (),
+	data = g_resource_lookup_data (resource,
 				       "/org/freedesktop/appstream/platform_env.txt",
 				       G_RESOURCE_LOOKUP_FLAGS_NONE,
 				       NULL);
@@ -1562,6 +1609,12 @@ as_utils_get_component_bundle_kind (AsComponent *cpt)
 	if (bundles->len > 0)
 		bundle_kind = as_bundle_get_kind (AS_BUNDLE (g_ptr_array_index (bundles, 0)));
 
+	/* assume "package" for system-wide components from metainfo files */
+	if (bundle_kind == AS_BUNDLE_KIND_UNKNOWN &&
+	    as_component_get_scope (cpt) == AS_COMPONENT_SCOPE_SYSTEM &&
+	    as_component_get_origin_kind (cpt) == AS_ORIGIN_KIND_METAINFO)
+		return AS_BUNDLE_KIND_PACKAGE;
+
 	return bundle_kind;
 }
 
@@ -1677,15 +1730,8 @@ as_sort_components_by_score (GPtrArray *cpts)
 void
 as_object_ptr_array_absorb (GPtrArray *dest, GPtrArray *src)
 {
-#if GLIB_CHECK_VERSION(2,58,0)
 	while (src->len != 0)
 		g_ptr_array_add (dest, g_ptr_array_steal_index_fast (src, 0));
-#else
-	while (src->len != 0) {
-		g_ptr_array_add (dest, g_object_ref (g_ptr_array_index (src, 0)));
-		g_ptr_array_remove_index_fast (src, 0);
-	}
-#endif
 }
 
 /**
@@ -1743,46 +1789,6 @@ as_filebasename_from_uri (const gchar *uri)
 	return bname;
 }
 
-/**
- * as_date_time_format_iso8601:
- * @datetime: A #GDateTime
- *
- * Format datetime in ISO 8601 format.
- *
- * Compatibility wrapper to support GLib < 2.62.
- * This function can go away if we bump the GLib minimal version.
- */
-gchar*
-as_date_time_format_iso8601 (GDateTime *datetime)
-{
-#if GLIB_CHECK_VERSION(2,62,0)
-	return g_date_time_format_iso8601 (datetime);
-#else
-	GString *outstr = NULL;
-	gchar *main_date = NULL;
-	gint64 offset;
-
-	/* Main date and time. */
-	main_date = g_date_time_format (datetime, "%Y-%m-%dT%H:%M:%S");
-	outstr = g_string_new (main_date);
-	g_free (main_date);
-
-	/* Timezone. Format it as `%:::z` unless the offset is zero, in which case
-	 * we can simply use `Z`. */
-	offset = g_date_time_get_utc_offset (datetime);
-
-	if (offset == 0) {
-		g_string_append_c (outstr, 'Z');
-	} else {
-		gchar *time_zone = g_date_time_format (datetime, "%:::z");
-		g_string_append (outstr, time_zone);
-		g_free (time_zone);
-	}
-
-	return g_string_free (outstr, FALSE);
-#endif
-}
-
 /**
  * as_strstripnl:
  * @string: a string to remove surrounding whitespaces and newlines
@@ -2040,7 +2046,7 @@ as_utils_install_icon_tarball (AsMetadataLocation location,
 				GError **error)
 {
 	g_autofree gchar *dir = NULL;
-	dir = g_strdup_printf ("%s%s/app-info/icons/%s/%s",
+	dir = g_strdup_printf ("%s%s/swcatalog/icons/%s/%s",
 			       destdir,
 			       as_metadata_location_get_prefix (location),
 			       origin,
@@ -2105,11 +2111,11 @@ as_utils_install_metadata_file (AsMetadataLocation location,
 	case AS_FORMAT_STYLE_COLLECTION:
 		if (g_strstr_len (filename, -1, ".yml.gz") != NULL) {
 			path = g_build_filename (as_metadata_location_get_prefix (location),
-						 "app-info", "yaml", NULL);
+						 "swcatalog", "yaml", NULL);
 			ret = as_utils_install_metadata_file_internal (filename, origin, path, destdir, TRUE, error);
 		} else {
 			path = g_build_filename (as_metadata_location_get_prefix (location),
-						 "app-info", "xmls", NULL);
+						 "swcatalog", "xml", NULL);
 			ret = as_utils_install_metadata_file_internal (filename, origin, path, destdir, FALSE, error);
 		}
 		break;
@@ -2184,16 +2190,14 @@ as_utils_install_metadata_file (AsMetadataLocation location,
  * Since: 0.14.2
  */
 gchar*
-as_get_user_cache_dir ()
+as_get_user_cache_dir (GError **error)
 {
 	const gchar *cache_root = g_get_user_cache_dir ();
 	if (cache_root == NULL) {
-		g_autoptr(GError) error = NULL;
-		gchar *tmp_dir = g_dir_make_tmp ("appstream-XXXXXX", &error);
+		gchar *tmp_dir = g_dir_make_tmp ("appstream-XXXXXX", error);
 		if (tmp_dir == NULL) {
 			/* something went very wrong here, we could neither get a user cache dir, nor
 			 * access to a temporary directory in /tmp */
-			g_error ("Unable to create temporary cache directory: %s", error->message);
 			return NULL;
 		}
 		return tmp_dir;
@@ -2243,3 +2247,194 @@ as_random_alnum_string (gssize len)
 
 	return ret;
 }
+
+/**
+ * as_utils_find_stock_icon_filename_full:
+ * @root_dir: the directory to search in, including prefix.
+ * @icon_name: the stock icon search name, e.g. "microphone.svg" or "kate"
+ * @icon_size: the icon color, e.g. 64 or 128. If size is 0, the first found icon is returned.
+ * @icon_scale the icon scaling factor, 1 for non HiDPI displays
+ * @error: a #GError or %NULL
+ *
+ * Finds an icon filename in the filesystem that matches the given specifications.
+ * This function may return a bigger icon than requested, which is suitable to be scaled down
+ * to the actually requested size.
+ * If no icon with the right scale factor is found, %NULL is returned.
+ *
+ * This algorithm does not implement the full Freedesktop icon theme specification,
+ * instead is is designed to find 99% of all application icons quickly and
+ * efficiently. It is not explicitly designed to find non-application stock
+ * icons as well.
+ * It also deliberately does not support legacy icon search locations and formats.
+ * It will however work on incomplete directory trees with missing icon theme definition files,
+ * by using a heuristic to find the right icon.
+ * If you need more features, and have a complete icon theme definition installed, use the
+ * icon-finding functions provided by GTK+ and Qt instead.
+ *
+ * If @icon_name is an absolute path, it will be returned unconditionally, as long
+ * as the icon it references exists on the filesystem.
+ *
+ * Returns: (transfer full): a newly allocated %NULL terminated string
+ *
+ * Since: 0.14.5
+ **/
+gchar*
+as_utils_find_stock_icon_filename_full (const gchar *root_dir,
+					const gchar *icon_name,
+					guint icon_size,
+					guint icon_scale,
+					GError **error)
+{
+	guint min_size_idx = 0;
+	const gchar *supported_ext[] = { ".png",
+					 ".svg",
+					 ".svgz",
+					 "",
+					 NULL };
+	const struct {
+		guint size;
+		const gchar *size_str;
+	} sizes[] =  {
+		{ 48,  "48x48" },
+		{ 64,  "64x64" },
+		{ 96,  "96x96" },
+		{ 128, "128x128" },
+		{ 256, "256x256" },
+		{ 512, "512x512" },
+		{ 0,   "scalable" },
+		{ 0,   NULL }
+	};
+	const gchar *types[] = { "actions",
+				 "animations",
+				 "apps",
+				 "categories",
+				 "devices",
+				 "emblems",
+				 "emotes",
+				 "filesystems",
+				 "intl",
+				 "mimetypes",
+				 "places",
+				 "status",
+				 "stock",
+				 NULL };
+	g_autofree gchar *prefix = NULL;
+
+	g_return_val_if_fail (icon_name != NULL, NULL);
+
+	/* fallbacks & sanitizations */
+	if (root_dir == NULL)
+		root_dir = "";
+	if (icon_scale <= 0)
+		icon_scale = 1;
+	if (icon_size > 512)
+		icon_size = 512;
+
+	/* is this an absolute path */
+	if (icon_name[0] == '/') {
+		g_autofree gchar *tmp = NULL;
+		tmp = g_build_filename (root_dir, icon_name, NULL);
+		if (!g_file_test (tmp, G_FILE_TEST_EXISTS)) {
+			g_set_error (error,
+				     AS_UTILS_ERROR,
+				     AS_UTILS_ERROR_FAILED,
+				     "specified icon '%s' does not exist",
+				     icon_name);
+			return NULL;
+		}
+		return g_strdup (tmp);
+	}
+
+	/* detect prefix */
+	prefix = g_build_filename (root_dir, "usr", NULL);
+	if (!g_file_test (prefix, G_FILE_TEST_EXISTS)) {
+		g_free (prefix);
+		prefix = g_strdup (root_dir);
+	}
+	if (!g_file_test (prefix, G_FILE_TEST_EXISTS)) {
+		g_set_error (error,
+			     AS_UTILS_ERROR,
+			     AS_UTILS_ERROR_FAILED,
+			     "Failed to find icon '%s' in %s", icon_name, prefix);
+		return NULL;
+	}
+
+	/* select minimum size */
+	for (guint i = 0; sizes[i].size_str != NULL; i++) {
+		if (sizes[i].size >= icon_size) {
+			min_size_idx = i;
+			break;
+		}
+	}
+
+	/* hicolor icon theme search */
+	for (guint i = min_size_idx; sizes[i].size_str != NULL; i++) {
+		g_autofree gchar *size = NULL;
+		if (icon_scale == 1)
+			size = g_strdup (sizes[i].size_str);
+		else
+			size = g_strdup_printf ("%s@%i", sizes[i].size_str, icon_scale);
+		for (guint m = 0; types[m] != NULL; m++) {
+			for (guint j = 0; supported_ext[j] != NULL; j++) {
+				g_autofree gchar *tmp = NULL;
+				tmp = g_strdup_printf ("%s/share/icons/"
+							"hicolor/%s/%s/%s%s",
+							prefix,
+							size,
+							types[m],
+							icon_name,
+							supported_ext[j]);
+				if (g_file_test (tmp, G_FILE_TEST_EXISTS))
+					return g_strdup (tmp);
+			}
+		}
+	}
+
+	/* breeze icon theme search, for KDE Plasma compatibility */
+	for (guint i = min_size_idx; sizes[i].size_str != NULL; i++) {
+		g_autofree gchar *size = NULL;
+		if (icon_scale == 1)
+			size = g_strdup (sizes[i].size_str);
+		else
+			size = g_strdup_printf ("%s@%i", sizes[i].size_str, icon_scale);
+		for (guint m = 0; types[m] != NULL; m++) {
+			for (guint j = 0; supported_ext[j] != NULL; j++) {
+				g_autofree gchar *tmp = NULL;
+				tmp = g_strdup_printf ("%s/share/icons/"
+							"breeze/%s/%s/%s%s",
+							prefix,
+							types[m],
+							size,
+							icon_name,
+							supported_ext[j]);
+				if (g_file_test (tmp, G_FILE_TEST_EXISTS))
+					return g_strdup (tmp);
+			}
+		}
+	}
+
+	/* failed */
+	g_set_error (error,
+		     AS_UTILS_ERROR,
+		     AS_UTILS_ERROR_FAILED,
+		     "Failed to find icon %s", icon_name);
+	return NULL;
+}
+
+/**
+ * as_utils_guess_scope_from_path:
+ * @path: The filename to test.
+ *
+ * Guess the #AsComponentScope that applies to a given path.
+ *
+ * Returns: the #AsComponentScope
+ *
+ * Since: 0.15.0
+ */
+AsComponentScope
+as_utils_guess_scope_from_path (const gchar *path)
+{
+	if (g_str_has_prefix (path, "/home") || g_str_has_prefix (path, g_get_home_dir ()))
+		return AS_COMPONENT_SCOPE_USER;
+	return AS_COMPONENT_SCOPE_SYSTEM;
+}
