@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2018-2021 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2018-2022 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -81,6 +81,11 @@ AsValidatorIssueTag as_validator_issue_tag_list[] =  {
 	     "and to provide more detailed information on this component immediately in the first paragraph.")
 	},
 
+	{ "description-first-word-not-capitalized",
+	  AS_ISSUE_SEVERITY_INFO,
+	  N_("The description line does not start with a capitalized word, project name or number.")
+	},
+
 	{ "description-has-plaintext-url",
 	  AS_ISSUE_SEVERITY_WARNING,
 	  N_("The description contains a web URL in plain text. This is not allowed, please use the <url/> tag instead to share links.")
@@ -119,6 +124,11 @@ AsValidatorIssueTag as_validator_issue_tag_list[] =  {
 	  N_("The component ID contains an invalid character. Only ASCII characters, dots and numbers are permitted.")
 	},
 
+	{ "cid-punctuation-prefix",
+	  AS_ISSUE_SEVERITY_ERROR,
+	  N_("The component ID starts with punctuation. This is not allowed.")
+	},
+
 	{ "cid-contains-hyphen",
 	  AS_ISSUE_SEVERITY_INFO,
 	  N_("The component ID contains a hyphen/minus. Using a hyphen is strongly discouraged, to improve interoperability with other tools such as D-Bus. "
@@ -133,7 +143,12 @@ AsValidatorIssueTag as_validator_issue_tag_list[] =  {
 
 	{ "cid-contains-uppercase-letter",
 	  AS_ISSUE_SEVERITY_PEDANTIC,
-	  N_("The component ID should only contain lowercase letters.")
+	  N_("The component ID should only contain lowercase characters.")
+	},
+
+	{ "cid-domain-not-lowercase",
+	  AS_ISSUE_SEVERITY_ERROR,
+	  N_("The domain part of the rDNS component ID (first two parts) must only contain lowercase characters.")
 	},
 
 	{ "cid-missing-affiliation-freedesktop",
@@ -273,7 +288,8 @@ AsValidatorIssueTag as_validator_issue_tag_list[] =  {
 
 	{ "relation-item-invalid-vercmp",
 	  AS_ISSUE_SEVERITY_ERROR,
-	  N_("Invalid comparison operation on relation item. Only eq/ne/lt/gt/le/ge are permitted.")
+	  /* TRANSLATORS: `eq/ne/lt/gt/le/ge` are AppStream XML values. Please do not translate them. */
+	  N_("Invalid comparison operation on relation item. Only one of `eq/ne/lt/gt/le/ge` is permitted.")
 	},
 
 	{ "relation-item-has-vercmp",
@@ -315,6 +331,12 @@ AsValidatorIssueTag as_validator_issue_tag_list[] =  {
 	     "make the item value refer to either the shortest or longest side of the display.")
 	},
 
+	{ "relation-hardware-value-invalid",
+	  AS_ISSUE_SEVERITY_WARNING,
+	  /* TRANSLATORS: Please do not translate AppStream tag and property names (in backticks). */
+	  N_("This `hardware` item contains an invalid value. It should be a Computer Hardware ID (CHID) UUID without braces.")
+	},
+
 	{ "component-type-invalid",
 	  AS_ISSUE_SEVERITY_ERROR,
 	  N_("The set component type is not a recognized, valid AppStream component type.")
@@ -372,7 +394,12 @@ AsValidatorIssueTag as_validator_issue_tag_list[] =  {
 
 	{ "summary-has-url",
 	  AS_ISSUE_SEVERITY_ERROR,
-	  N_("The summary must not contain any URL. Use the <url/> tags for links.")
+	  N_("The summary must not contain any URL. Use the `<url/>` tags for links.")
+	},
+
+	{ "summary-first-word-not-capitalized",
+	  AS_ISSUE_SEVERITY_INFO,
+	  N_("The summary text does not start with a capitalized word, project name or number.")
 	},
 
 	{ "icon-stock-cached-has-url",
@@ -410,7 +437,7 @@ AsValidatorIssueTag as_validator_issue_tag_list[] =  {
 	  N_("Invalid `type` property for this `url` tag. URLs of this type are not known in the AppStream specification."),
 	},
 
-	{ "url-not-found",
+	{ "url-not-reachable",
 	  AS_ISSUE_SEVERITY_WARNING,
 	  N_("Unable to reach remote location that this URL references - does it exist?"),
 	},
@@ -576,7 +603,7 @@ AsValidatorIssueTag as_validator_issue_tag_list[] =  {
 
 	{ "screenshot-caption-too-long",
 	  AS_ISSUE_SEVERITY_PEDANTIC,
-	  N_("The screenshot caption is too long (should be <= 80 characters)"),
+	  N_("The screenshot caption is too long (should be <= 100 characters)"),
 	},
 
 	{ "file-read-failed",
@@ -663,6 +690,17 @@ AsValidatorIssueTag as_validator_issue_tag_list[] =  {
 	  N_("The value set as release type is invalid."),
 	},
 
+	{ "release-version-missing",
+	  AS_ISSUE_SEVERITY_ERROR,
+	  /* TRANSLATORS: Please do not translate AppStream tag and property names (in backticks). */
+	  N_("The release is missing the `version` property."),
+	},
+
+	{ "release-time-missing",
+	  AS_ISSUE_SEVERITY_ERROR,
+	  /* TRANSLATORS: Please do not translate AppStream tag and property names (in backticks). */
+	  N_("The release is missing either the `date` (preferred) or the `timestamp` property."),
+	},
 	{ "artifact-type-invalid",
 	  AS_ISSUE_SEVERITY_ERROR,
 	  /* TRANSLATORS: Please do not translate AppStream tag and property names (in backticks). */
@@ -756,6 +794,33 @@ AsValidatorIssueTag as_validator_issue_tag_list[] =  {
 	     "You can generate the tag data online by answering a few questions at https://hughsie.github.io/oars/"),
 	},
 
+	{ "component-tag-missing-namespace",
+	  AS_ISSUE_SEVERITY_ERROR,
+	  /* TRANSLATORS: Please do not translate AppStream tag and property names (in backticks) and keep the URL intact. */
+	  N_("This `tag` is missing a `namespace` attribute."),
+	},
+
+	{ "component-tag-invalid",
+	  AS_ISSUE_SEVERITY_ERROR,
+	  /* TRANSLATORS: Please do not translate AppStream tag and property names (in backticks) and keep the URL intact. */
+	  N_("This tag or its namespace contains invalid characters. Only lower-cased ASCII letters, numbers, dots, hyphens and underscores are permitted."),
+	},
+
+	{ "branding-color-type-invalid",
+	  AS_ISSUE_SEVERITY_ERROR,
+	  N_("The type of this color is not valid."),
+	},
+
+	{ "branding-color-scheme-type-invalid",
+	  AS_ISSUE_SEVERITY_ERROR,
+	  N_("The value of this color scheme preference is not valid."),
+	},
+
+	{ "branding-color-invalid",
+	  AS_ISSUE_SEVERITY_ERROR,
+	  N_("This color is not a valid HTML color code."),
+	},
+
 	{ NULL, AS_ISSUE_SEVERITY_UNKNOWN, NULL }
 };
 
