@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2014-2021 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2014-2022 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -349,7 +349,7 @@ as_validator_check_web_url (AsValidator *validator, xmlNode *node, const gchar *
 	if (!priv->check_urls)
 		return TRUE;
 
-	g_debug ("Checking URL availability: %s\n", url);
+	g_debug ("Checking URL availability: %s", url);
 
 	/* try to download first few bytes of the file, get error if that fails */
 	if (!as_curl_check_url_exists (priv->acurl, url, &tmp_error)) {
@@ -520,7 +520,7 @@ as_validator_check_children_quick (AsValidator *validator, xmlNode *node, const
 						"invalid-child-tag-name",
 						/* TRANSLATORS: An invalid XML tag was found, "Found" refers to the tag name found, "Allowed" to the permitted name. */
 						_("Found: %s - Allowed: %s"),
-						(const gchar*) node->name,
+						node_name,
 						allowed_tagname);
 		}
 	}
@@ -543,6 +543,42 @@ as_validator_check_nolocalized (AsValidator *validator, xmlNode* node, const gch
 	}
 }
 
+/**
+ * as_validator_first_word_capitalized:
+ */
+static gboolean
+as_validator_first_word_capitalized (AsValidator *validator, const gchar *text)
+{
+	AsValidatorPrivate *priv = GET_PRIVATE (validator);
+	g_autofree gchar *first_word = NULL;
+	gchar *tmp;
+
+	if (text == NULL || text[0] == '\0')
+		return TRUE;
+
+	/* text starts with a number, that's fine */
+	if (g_ascii_isdigit (text[0]))
+		return TRUE;
+
+	/* get the first word */
+	first_word = g_strdup (text);
+	tmp = g_strstr_len (first_word, -1, " ");
+	if (tmp != NULL)
+		*tmp = '\0';
+
+	/* we accept a capitalization anywhere in the first word */
+	for (guint i = 0; first_word[i] != '\0'; i++) {
+		if (first_word[i] >= 'A' && first_word[i] <= 'Z')
+			return TRUE;
+	}
+
+	/* if the first word is the project's name, we accept whatever spelling they prefer */
+	if (g_strcmp0 (first_word, as_component_get_name (priv->current_cpt)) == 0)
+		return TRUE;
+
+	return FALSE;
+}
+
 /**
  * as_validator_check_description_paragraph:
  **/
@@ -601,12 +637,16 @@ static void
 as_validator_check_description_tag (AsValidator *validator, xmlNode* node, AsFormatStyle mode, gboolean main_description)
 {
 	gboolean first_paragraph = TRUE;
+	gboolean is_localized = FALSE;
 
 	if (mode == AS_FORMAT_STYLE_METAINFO) {
 		as_validator_check_nolocalized (validator,
 						node,
 						"metainfo-localized-description-tag",
 						(const gchar*) node->name);
+	} else {
+		g_autofree gchar *lang = as_xml_get_prop_value (node, "lang");
+		is_localized = lang != NULL;
 	}
 
 	for (xmlNode *iter = node->children; iter != NULL; iter = iter->next) {
@@ -624,6 +664,8 @@ as_validator_check_description_tag (AsValidator *validator, xmlNode* node, AsFor
 		}
 
 		if (g_strcmp0 (node_name, "p") == 0) {
+			g_autofree gchar *p_content = as_strstripnl ((gchar*) xmlNodeGetContent (iter));
+
 			if (mode == AS_FORMAT_STYLE_COLLECTION) {
 				as_validator_check_nolocalized (validator,
 								iter,
@@ -641,6 +683,20 @@ as_validator_check_description_tag (AsValidator *validator, xmlNode* node, AsFor
 			}
 			first_paragraph = FALSE;
 
+			/* in metainfo mode, we need to check every node for localization,
+			 * otherwise we just honor the is_localized var */
+			if (mode == AS_FORMAT_STYLE_METAINFO) {
+				g_autofree gchar *lang = as_xml_get_prop_value (iter, "lang");
+				is_localized = lang != NULL;
+			}
+
+			/* validate spelling */
+			if (!is_localized && !as_validator_first_word_capitalized (validator, p_content))
+				as_validator_add_issue (validator, node,
+							"description-first-word-not-capitalized",
+							NULL);
+
+			/* validate common stuff */
 			as_validator_check_description_paragraph (validator, iter);
 		} else if (g_strcmp0 (node_name, "ul") == 0) {
 			if (mode == AS_FORMAT_STYLE_COLLECTION) {
@@ -700,6 +756,16 @@ as_validator_check_appear_once (AsValidator *validator, xmlNode *node, GHashTabl
 	g_hash_table_add (known_tags, tag_id);
 }
 
+static gboolean
+as_validate_string_lowercase (const gchar *str)
+{
+	for (guint i = 0; str[i] != '\0'; i++) {
+		if (g_ascii_isalpha (str[i]) && g_ascii_isupper (str[i]))
+			return FALSE;
+	}
+	return TRUE;
+}
+
 /**
  * as_validator_validate_component_id:
  *
@@ -708,10 +774,13 @@ as_validator_check_appear_once (AsValidator *validator, xmlNode *node, GHashTabl
 static void
 as_validator_validate_component_id (AsValidator *validator, xmlNode *idnode, AsComponent *cpt)
 {
-	guint i;
 	g_auto(GStrv) cid_parts = NULL;
 	gboolean hyphen_found = FALSE;
 	g_autofree gchar *cid = (gchar*) xmlNodeGetContent (idnode);
+	g_return_if_fail (cid != NULL);
+
+	if (cid[0] != '\0' && g_ascii_ispunct (cid[0]))
+		as_validator_add_issue (validator, idnode, "cid-punctuation-prefix", cid);
 
 	cid_parts = g_strsplit (cid, ".", -1);
 	if (g_strv_length (cid_parts) < 3) {
@@ -729,10 +798,16 @@ as_validator_validate_component_id (AsValidator *validator, xmlNode *idnode, AsC
 		if (!as_utils_is_tld (cid_parts[0])) {
 			as_validator_add_issue (validator, idnode, "cid-maybe-not-rdns", cid);
 		}
+
+		/* ensure first parts of the rDNS ID are always lowercase */
+		if (!as_validate_string_lowercase (cid_parts[0]))
+			as_validator_add_issue (validator, idnode, "cid-domain-not-lowercase", cid);
+		if (!as_validate_string_lowercase (cid_parts[1]))
+			as_validator_add_issue (validator, idnode, "cid-domain-not-lowercase", cid);
 	}
 
 	/* validate characters in AppStream ID */
-	for (i = 0; cid[i] != '\0'; i++) {
+	for (guint i = 0; cid[i] != '\0'; i++) {
 		/* check if we have a printable, alphanumeric ASCII character or a dot, hyphen or underscore */
 		if ((!g_ascii_isalnum (cid[i])) &&
 		    (cid[i] != '.') &&
@@ -742,11 +817,12 @@ as_validator_validate_component_id (AsValidator *validator, xmlNode *idnode, AsC
 			c = g_utf8_substring (cid, i, i + 1);
 			as_validator_add_issue (validator, idnode,
 						"cid-invalid-character",
-						"%s: '%c'", cid, c);
+						"%s: '%s'", cid, c);
 		}
 
 		if (!hyphen_found && cid[i] == '-') {
 			hyphen_found = TRUE;
+			/* a hyphen in the ID is bad news, because we can't use the ID on DBus and it also clashes with other naming schemes */
 			as_validator_add_issue (validator, idnode, "cid-contains-hyphen", cid);
 		}
 
@@ -755,7 +831,7 @@ as_validator_validate_component_id (AsValidator *validator, xmlNode *idnode, AsC
 	}
 
 	/* check if any segment starts with a number */
-	for (i = 0; cid_parts[i] != NULL; i++) {
+	for (guint i = 0; cid_parts[i] != NULL; i++) {
 		if (g_ascii_isdigit (cid_parts[i][0])) {
 			as_validator_add_issue (validator, idnode,
 						"cid-has-number-prefix",
@@ -764,11 +840,6 @@ as_validator_validate_component_id (AsValidator *validator, xmlNode *idnode, AsC
 		}
 	}
 
-
-	/* a hyphen in the ID is bad news, because we can't use the ID on DBus and it also clashes with other naming schemes */
-	if (g_strstr_len (cid, -1, "-") != NULL) {
-	}
-
 	/* project-group specific constraints on the ID */
 	if ((g_strcmp0 (as_component_get_project_group (cpt), "Freedesktop") == 0) ||
 	    (g_strcmp0 (as_component_get_project_group (cpt), "FreeDesktop") == 0)) {
@@ -907,6 +978,76 @@ as_validator_validate_update_contact (AsValidator *validator, xmlNode *uc_node)
 	}
 }
 
+/**
+ * as_id_string_valid:
+ */
+gboolean
+as_id_string_valid (const gchar *str, gboolean allow_uppercase)
+{
+	if (str == NULL)
+		return FALSE;
+
+	for (guint i = 0; str[i] != '\0'; i++) {
+		/* check if we have a printable, alphanumeric ASCII character or a dot, hyphen or underscore */
+		if ((!g_ascii_isalnum (str[i])) &&
+		    (str[i] != '.') &&
+		    (str[i] != '-') &&
+		    (str[i] != '_')) {
+			/* found a character that is not whitelisted */
+			return FALSE;
+		}
+
+		if (!allow_uppercase && g_ascii_isalpha (str[i]) && g_ascii_isupper (str[i]))
+			return FALSE;
+	}
+
+	return TRUE;
+}
+
+/**
+ * as_validator_check_tags:
+ **/
+static void
+as_validator_check_tags (AsValidator *validator, xmlNode *node)
+{
+	for (xmlNode *iter = node->children; iter != NULL; iter = iter->next) {
+		const gchar *node_name;
+		g_autofree gchar *ns = NULL;
+		g_autofree gchar *value = NULL;
+
+		/* discard spaces */
+		if (iter->type != XML_ELEMENT_NODE)
+			continue;
+		node_name = (const gchar*) iter->name;
+
+		if (g_strcmp0 (node_name, "tag") != 0) {
+			as_validator_add_issue (validator, node,
+						"invalid-child-tag-name",
+						/* TRANSLATORS: An invalid XML tag was found, "Found" refers to the tag name found, "Allowed" to the permitted name. */
+						_("Found: %s - Allowed: %s"),
+						(const gchar*) node->name,
+						"tag");
+			continue;
+		}
+		as_validator_check_content_empty (validator, iter, "tags/tag");
+
+		ns = as_xml_get_prop_value (iter, "namespace");
+		if (ns == NULL) {
+			as_validator_add_issue (validator, iter, "component-tag-missing-namespace", NULL);
+			continue;
+		}
+
+		if (!as_id_string_valid (ns, FALSE)) {
+			as_validator_add_issue (validator, iter, "component-tag-invalid", ns);
+			continue;
+		}
+
+		value = as_xml_get_node_value (iter);
+		if (!as_id_string_valid (value, FALSE))
+			as_validator_add_issue (validator, iter, "component-tag-invalid", value);
+	}
+}
+
 /**
  * as_validator_check_screenshots:
  *
@@ -1045,10 +1186,10 @@ as_validator_check_screenshots (AsValidator *validator, xmlNode *node, AsCompone
 }
 
 /**
- * as_validator_check_requires_recommends:
+ * as_validator_check_relations:
  **/
 static void
-as_validator_check_requires_recommends (AsValidator *validator, xmlNode *node, AsComponent *cpt, AsRelationKind kind)
+as_validator_check_relations (AsValidator *validator, xmlNode *node, AsComponent *cpt, AsRelationKind kind)
 {
 	for (xmlNode *iter = node->children; iter != NULL; iter = iter->next) {
 		const gchar *node_name;
@@ -1095,6 +1236,7 @@ as_validator_check_requires_recommends (AsValidator *validator, xmlNode *node, A
 		switch (item_kind) {
 		case AS_RELATION_ITEM_KIND_MODALIAS:
 		case AS_RELATION_ITEM_KIND_CONTROL:
+		case AS_RELATION_ITEM_KIND_HARDWARE:
 			can_have_version = FALSE;
 			can_have_compare = FALSE;
 			break;
@@ -1161,6 +1303,19 @@ as_validator_check_requires_recommends (AsValidator *validator, xmlNode *node, A
 			if (as_display_side_kind_from_string (side_str) == AS_DISPLAY_SIDE_KIND_UNKNOWN)
 				as_validator_add_issue (validator, iter, "relation-display-length-side-property-invalid", side_str);
 		}
+
+		/* check hardware for sanity */
+		if (item_kind == AS_RELATION_ITEM_KIND_HARDWARE) {
+			guint dash_count = 0;
+			for (guint i = 0; content[i] != '\0'; i++)
+			     if (content[i] == '-')
+				     dash_count++;
+
+			if (g_str_has_prefix (content, "{") || g_str_has_suffix (content, "}"))
+				as_validator_add_issue (validator, iter, "relation-hardware-value-invalid", content);
+			else if (dash_count != 4)
+				as_validator_add_issue (validator, iter, "relation-hardware-value-invalid", content);
+		}
 	}
 }
 
@@ -1364,11 +1519,22 @@ as_validator_check_release (AsValidator *validator, xmlNode *node, AsFormatStyle
 {
 	gchar *prop;
 
+	/* validate presence of version property */
+	prop = as_xml_get_prop_value (node, "version");
+	if (prop == NULL)
+		as_validator_add_issue (validator, node, "release-version-missing", "version");
+	g_free (prop);
+
 	/* validate date strings */
 	prop = as_xml_get_prop_value (node, "date");
 	if (prop != NULL) {
 		as_validator_validate_iso8601_complete_date (validator, node, prop);
 		g_free (prop);
+	} else {
+		g_autofree gchar *timestamp = as_xml_get_prop_value (node, "timestamp");
+		/* Neither timestamp, nor date property exists */
+		if (timestamp == NULL)
+			as_validator_add_issue (validator, node, "release-time-missing", "date");
 	}
 	prop = as_xml_get_prop_value (node, "date_eol");
 	if (prop != NULL) {
@@ -1489,6 +1655,45 @@ as_validator_check_releases (AsValidator *validator, xmlNode *node, AsFormatStyl
 	}
 }
 
+/**
+ * as_validator_check_branding:
+ **/
+static void
+as_validator_check_branding (AsValidator *validator, xmlNode *node)
+{
+	for (xmlNode *iter = node->children; iter != NULL; iter = iter->next) {
+		if (iter->type != XML_ELEMENT_NODE)
+			continue;
+
+		if (g_strcmp0 ((gchar*) iter->name, "color") == 0) {
+			gchar *tmp;
+			guint len;
+
+			tmp = as_xml_get_prop_value (iter, "type");
+			if (as_color_kind_from_string (tmp) == AS_COLOR_KIND_UNKNOWN)
+				as_validator_add_issue (validator, iter, "branding-color-type-invalid", tmp);
+			g_free (tmp);
+
+			tmp = as_xml_get_prop_value (iter, "scheme_preference");
+			if (tmp != NULL && as_color_scheme_kind_from_string (tmp) == AS_COLOR_SCHEME_KIND_UNKNOWN)
+				as_validator_add_issue (validator, iter, "branding-color-scheme-type-invalid", tmp);
+			g_free (tmp);
+
+			tmp = as_xml_get_node_value (iter);
+			len = strlen (tmp);
+			if (!g_str_has_prefix (tmp, "#") || (len != 7 && len != 9))
+				as_validator_add_issue (validator, iter, "branding-color-invalid", tmp);
+		} else {
+			as_validator_add_issue (validator, iter,
+						"invalid-child-tag-name",
+						/* TRANSLATORS: An invalid XML tag was found, "Found" refers to the tag name found, "Allowed" to the permitted names. */
+						_("Found: %s - Allowed: %s"),
+						(const gchar*) iter->name,
+						"color");
+		}
+	}
+}
+
 /**
  * as_validator_validate_component_node:
  **/
@@ -1498,6 +1703,7 @@ as_validator_validate_component_node (AsValidator *validator, AsContext *ctx, xm
 	AsComponent *cpt;
 	g_autofree gchar *cpttype = NULL;
 	g_autoptr(GHashTable) found_tags = NULL;
+	g_autofree gchar *date_eol_str = NULL;
 
 	AsFormatStyle mode;
 	gboolean has_metadata_license = FALSE;
@@ -1508,6 +1714,7 @@ as_validator_validate_component_node (AsValidator *validator, AsContext *ctx, xm
 	/* validate the resulting AsComponent for sanity */
 	cpt = as_component_new ();
 	as_component_load_from_xml (cpt, ctx, root, NULL);
+	as_component_set_active_locale (cpt, "C");
 	as_validator_set_current_cpt (validator, cpt);
 
 	/* check if component type is valid */
@@ -1520,6 +1727,12 @@ as_validator_validate_component_node (AsValidator *validator, AsContext *ctx, xm
 		}
 	}
 
+	/* validate EOL date format */
+	date_eol_str = as_xml_get_prop_value (root, "date_eol");
+	if (date_eol_str != NULL)
+		as_validator_validate_iso8601_complete_date (validator, root, date_eol_str);
+
+	/* validate other component properties */
 	if ((as_component_get_priority (cpt) != 0) && (mode == AS_FORMAT_STYLE_METAINFO))
 		as_validator_add_issue (validator, root, "component-priority-in-metainfo", NULL);
 
@@ -1555,6 +1768,8 @@ as_validator_validate_component_node (AsValidator *validator, AsContext *ctx, xm
 			continue;
 		node_name = (const gchar*) iter->name;
 		node_content = (gchar*) xmlNodeGetContent (iter);
+		if (node_content != NULL)
+			node_content = as_strstripnl (node_content);
 
 		if (g_strcmp0 (node_name, "id") == 0) {
 			g_autofree gchar *prop = as_xml_get_prop_value (iter, (xmlChar*) "type");
@@ -1582,6 +1797,7 @@ as_validator_validate_component_node (AsValidator *validator, AsContext *ctx, xm
 				as_validator_add_issue (validator, iter, "name-has-dot-suffix", node_content);
 
 		} else if (g_strcmp0 (node_name, "summary") == 0) {
+			g_autofree gchar *lang = NULL;
 			const gchar *summary = node_content;
 
 			as_validator_check_appear_once (validator, iter, found_tags);
@@ -1600,6 +1816,12 @@ as_validator_validate_component_node (AsValidator *validator, AsContext *ctx, xm
 							node_name);
 			}
 
+			lang = as_xml_get_prop_value (iter, "lang");
+			if (lang == NULL && !as_validator_first_word_capitalized (validator, summary))
+				as_validator_add_issue (validator, iter,
+							"summary-first-word-not-capitalized",
+							NULL);
+
 		} else if (g_strcmp0 (node_name, "description") == 0) {
 			as_validator_check_appear_once (validator, iter, found_tags);
 			as_validator_check_description_tag (validator, iter, mode, TRUE);
@@ -1645,7 +1867,7 @@ as_validator_validate_component_node (AsValidator *validator, AsContext *ctx, xm
 			as_validator_check_web_url (validator,
 						    iter,
 						    node_content,
-						    "url-not-found");
+						    "url-not-reachable");
 
 		} else if (g_strcmp0 (node_name, "categories") == 0) {
 			as_validator_check_appear_once (validator, iter, found_tags);
@@ -1726,11 +1948,19 @@ as_validator_validate_component_node (AsValidator *validator, AsContext *ctx, xm
 			as_validator_check_children_quick (validator, iter, "content_attribute", TRUE);
 			can_be_empty = TRUE;
 		} else if (g_strcmp0 (node_name, "requires") == 0) {
-			as_validator_check_requires_recommends (validator, iter, cpt, AS_RELATION_KIND_REQUIRES);
+			as_validator_check_relations (validator, iter, cpt, AS_RELATION_KIND_REQUIRES);
 		} else if (g_strcmp0 (node_name, "recommends") == 0) {
-			as_validator_check_requires_recommends (validator, iter, cpt, AS_RELATION_KIND_RECOMMENDS);
+			as_validator_check_relations (validator, iter, cpt, AS_RELATION_KIND_RECOMMENDS);
+		} else if (g_strcmp0 (node_name, "supports") == 0) {
+			as_validator_check_relations (validator, iter, cpt, AS_RELATION_KIND_SUPPORTS);
 		} else if (g_strcmp0 (node_name, "agreement") == 0) {
 			as_validator_check_children_quick (validator, iter, "agreement_section", FALSE);
+		} else if (g_strcmp0 (node_name, "branding") == 0) {
+			as_validator_check_appear_once (validator, iter, found_tags);
+			as_validator_check_branding (validator, iter);
+		} else if (g_strcmp0 (node_name, "tags") == 0) {
+			as_validator_check_appear_once (validator, iter, found_tags);
+			as_validator_check_tags (validator, iter);
 		} else if (g_strcmp0 (node_name, "name_variant_suffix") == 0) {
 			as_validator_check_appear_once (validator, iter, found_tags);
 		} else if (g_strcmp0 (node_name, "custom") == 0) {
@@ -1891,7 +2121,7 @@ as_validator_validate_component_node (AsValidator *validator, AsContext *ctx, xm
 			AsScreenshot *scr = AS_SCREENSHOT (g_ptr_array_index (scr_array, j));
 			const gchar *scr_caption = as_screenshot_get_caption (scr);
 
-			if ((scr_caption != NULL) && (strlen (scr_caption) > 80)) {
+			if ((scr_caption != NULL) && (strlen (scr_caption) > 100)) {
 				as_validator_add_issue (validator, NULL,
 							"screenshot-caption-too-long",
 							scr_caption);
