@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2012-2021 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2012-2022 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -63,7 +63,8 @@ gchar		*as_xml_dump_node_children (xmlNode *node);
 
 void		as_xml_add_description_node (AsContext *ctx,
 					     xmlNode *root,
-					     GHashTable *desc_table);
+					     GHashTable *desc_table,
+					     gboolean mi_translatable);
 xmlNode		*as_xml_add_description_node_raw (xmlNode *root,
 						  const gchar *description);
 
