@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2012-2021 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2012-2022 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -192,6 +192,7 @@ test_appstream_parser_locale ()
 	g_assert_cmpint (trs->len, ==, 1);
 	tr = AS_TRANSLATION (g_ptr_array_index (trs, 0));
 	g_assert_cmpstr (as_translation_get_id (tr), ==, "firefox");
+	g_assert_cmpstr (as_translation_get_source_locale (tr), ==, "de");
 
 	/* check if we loaded the right amount of icons */
 	g_assert_cmpint (as_component_get_icons (cpt)->len, ==, 2);
@@ -236,7 +237,7 @@ test_appstream_write_locale ()
 				    "    <mediatype>x-scheme-handler/http</mediatype>\n"
 				    "    <mediatype>x-scheme-handler/https</mediatype>\n"
 				    "  </provides>\n"
-				    "  <translation type=\"gettext\">firefox</translation>\n"
+				    "  <translation type=\"gettext\" source_locale=\"de\">firefox</translation>\n"
 				    "  <keywords>\n"
 				    "    <keyword>internet</keyword>\n"
 				    "    <keyword>web</keyword>\n"
@@ -596,7 +597,7 @@ test_appstream_read_description (void)
 								 "<p>Paragraph</p>\n");
 }
 
-static const gchar *xmldata_simple =    "<component>\n"
+static const gchar *xmldata_simple =    "<component date_eol=\"2022-02-22T00:00:00Z\">\n"
 					"  <id>org.example.SimpleTest</id>\n"
 					"  <name>TestComponent</name>\n"
 					"  <name_variant_suffix>Generic</name_variant_suffix>\n"
@@ -619,6 +620,8 @@ test_xml_read_simple (void)
 	g_assert_cmpstr (as_component_get_name (cpt), ==, "TestComponent");
 	g_assert_cmpstr (as_component_get_summary (cpt), ==, "Just part of an unittest");
 	g_assert_cmpstr (as_component_get_name_variant_suffix (cpt), ==, "Generic");
+	g_assert_cmpstr (as_component_get_date_eol (cpt), ==, "2022-02-22T00:00:00Z");
+	g_assert_cmpint (as_component_get_timestamp_eol (cpt), ==, 1645488000);
 }
 
 /**
@@ -635,6 +638,7 @@ test_xml_write_simple (void)
 	cpt = as_component_new ();
 	as_component_set_kind (cpt, AS_COMPONENT_KIND_GENERIC);
 	as_component_set_id (cpt, "org.example.SimpleTest");
+	as_component_set_date_eol (cpt, "2022-02-22");
 
 	as_component_set_name (cpt, "TestComponent", "C");
 	as_component_set_summary (cpt, "Just part of an unittest", "C");
@@ -1399,40 +1403,47 @@ test_xml_write_screenshots (void)
 }
 
 
-static const gchar *xmldata_recommends_requires = "<component>\n"
-						  "  <id>org.example.RelationsTest</id>\n"
-						  "  <requires>\n"
-						  "    <kernel version=\"4.15\" compare=\"ge\">Linux</kernel>\n"
-						  "    <id version=\"1.2\" compare=\"eq\">org.example.TestDependency</id>\n"
-						  "    <display_length>small</display_length>\n"
-						  "  </requires>\n"
-						  "  <recommends>\n"
-						  "    <memory>2500</memory>\n"
-						  "    <modalias>usb:v1130p0202d*</modalias>\n"
-						  "    <display_length side=\"longest\" compare=\"le\">4200</display_length>\n"
-						  "  </recommends>\n"
-						  "</component>\n";
+static const gchar *xmldata_relations = "<component>\n"
+					"  <id>org.example.RelationsTest</id>\n"
+					"  <requires>\n"
+					"    <kernel version=\"4.15\" compare=\"ge\">Linux</kernel>\n"
+					"    <id version=\"1.2\" compare=\"eq\">org.example.TestDependency</id>\n"
+					"    <display_length>small</display_length>\n"
+					"  </requires>\n"
+					"  <recommends>\n"
+					"    <memory>2500</memory>\n"
+					"    <modalias>usb:v1130p0202d*</modalias>\n"
+					"    <display_length side=\"longest\" compare=\"le\">4200</display_length>\n"
+					"  </recommends>\n"
+					"  <supports>\n"
+					"    <control>gamepad</control>\n"
+					"    <control>keyboard</control>\n"
+					"  </supports>\n"
+					"</component>\n";
 /**
- * test_xml_read_recommends_requires:
+ * test_xml_read_relations:
  *
- * Test reading the recommends/requires tags.
+ * Test reading the recommends/requires/supports tags.
  */
 static void
-test_xml_read_recommends_requires (void)
+test_xml_read_relations (void)
 {
 	g_autoptr(AsComponent) cpt = NULL;
 	GPtrArray *recommends;
 	GPtrArray *requires;
+	GPtrArray *supports;
 	AsRelation *relation;
 
-	cpt = as_xml_test_read_data (xmldata_recommends_requires, AS_FORMAT_STYLE_METAINFO);
+	cpt = as_xml_test_read_data (xmldata_relations, AS_FORMAT_STYLE_METAINFO);
 	g_assert_cmpstr (as_component_get_id (cpt), ==, "org.example.RelationsTest");
 
 	recommends = as_component_get_recommends (cpt);
 	requires = as_component_get_requires (cpt);
+	supports = as_component_get_supports (cpt);
 
-	g_assert_cmpint (recommends->len, ==, 3);
 	g_assert_cmpint (requires->len, ==, 3);
+	g_assert_cmpint (recommends->len, ==, 3);
+	g_assert_cmpint (supports->len, ==, 2);
 
 	/* memory relation */
 	relation = AS_RELATION (g_ptr_array_index (recommends, 0));
@@ -1475,15 +1486,25 @@ test_xml_read_recommends_requires (void)
 	g_assert_cmpint (as_relation_get_item_kind (relation), ==, AS_RELATION_ITEM_KIND_DISPLAY_LENGTH);
 	g_assert_cmpint (as_relation_get_value_display_length_kind (relation), ==, AS_DISPLAY_LENGTH_KIND_SMALL);
 	g_assert_cmpint (as_relation_get_compare (relation), ==, AS_RELATION_COMPARE_GE);
+
+	/* control relation */
+	relation = AS_RELATION (g_ptr_array_index (supports, 0));
+	g_assert_cmpint (as_relation_get_kind (relation), ==, AS_RELATION_KIND_SUPPORTS);
+	g_assert_cmpint (as_relation_get_item_kind (relation), ==, AS_RELATION_ITEM_KIND_CONTROL);
+	g_assert_cmpint (as_relation_get_value_control_kind (relation), ==, AS_CONTROL_KIND_GAMEPAD);
+	relation = AS_RELATION (g_ptr_array_index (supports, 1));
+	g_assert_cmpint (as_relation_get_kind (relation), ==, AS_RELATION_KIND_SUPPORTS);
+	g_assert_cmpint (as_relation_get_item_kind (relation), ==, AS_RELATION_ITEM_KIND_CONTROL);
+	g_assert_cmpint (as_relation_get_value_control_kind (relation), ==, AS_CONTROL_KIND_KEYBOARD);
 }
 
 /**
- * test_xml_write_recommends_requires:
+ * test_xml_write_relations:
  *
- * Test writing the recommends/requires tags.
+ * Test writing the recommends/requires/supports tags.
  */
 static void
-test_xml_write_recommends_requires (void)
+test_xml_write_relations (void)
 {
 	g_autoptr(AsComponent) cpt = NULL;
 	g_autofree gchar *res = NULL;
@@ -1493,6 +1514,8 @@ test_xml_write_recommends_requires (void)
 	g_autoptr(AsRelation) id_relation = NULL;
 	g_autoptr(AsRelation) dl_relation1 = NULL;
 	g_autoptr(AsRelation) dl_relation2 = NULL;
+	g_autoptr(AsRelation) ctl_relation1 = NULL;
+	g_autoptr(AsRelation) ctl_relation2 = NULL;
 
 	cpt = as_component_new ();
 	as_component_set_id (cpt, "org.example.RelationsTest");
@@ -1503,6 +1526,8 @@ test_xml_write_recommends_requires (void)
 	id_relation = as_relation_new ();
 	dl_relation1 = as_relation_new ();
 	dl_relation2 = as_relation_new ();
+	ctl_relation1 = as_relation_new ();
+	ctl_relation2 = as_relation_new ();
 
 	as_relation_set_kind (mem_relation, AS_RELATION_KIND_RECOMMENDS);
 	as_relation_set_kind (moda_relation, AS_RELATION_KIND_RECOMMENDS);
@@ -1510,6 +1535,8 @@ test_xml_write_recommends_requires (void)
 	as_relation_set_kind (id_relation, AS_RELATION_KIND_REQUIRES);
 	as_relation_set_kind (dl_relation1, AS_RELATION_KIND_RECOMMENDS);
 	as_relation_set_kind (dl_relation2, AS_RELATION_KIND_REQUIRES);
+	as_relation_set_kind (ctl_relation1, AS_RELATION_KIND_SUPPORTS);
+	as_relation_set_kind (ctl_relation2, AS_RELATION_KIND_SUPPORTS);
 
 	as_relation_set_item_kind (mem_relation, AS_RELATION_ITEM_KIND_MEMORY);
 	as_relation_set_value_int (mem_relation, 2500);
@@ -1535,15 +1562,22 @@ test_xml_write_recommends_requires (void)
 	as_relation_set_value_display_length_kind (dl_relation2, AS_DISPLAY_LENGTH_KIND_SMALL);
 	as_relation_set_compare (dl_relation2, AS_RELATION_COMPARE_GE);
 
+	as_relation_set_item_kind (ctl_relation1, AS_RELATION_ITEM_KIND_CONTROL);
+	as_relation_set_item_kind (ctl_relation2, AS_RELATION_ITEM_KIND_CONTROL);
+	as_relation_set_value_control_kind (ctl_relation1, AS_CONTROL_KIND_GAMEPAD);
+	as_relation_set_value_control_kind (ctl_relation2, AS_CONTROL_KIND_KEYBOARD);
+
 	as_component_add_relation (cpt, mem_relation);
 	as_component_add_relation (cpt, moda_relation);
 	as_component_add_relation (cpt, kernel_relation);
 	as_component_add_relation (cpt, id_relation);
 	as_component_add_relation (cpt, dl_relation1);
 	as_component_add_relation (cpt, dl_relation2);
+	as_component_add_relation (cpt, ctl_relation1);
+	as_component_add_relation (cpt, ctl_relation2);
 
 	res = as_xml_test_serialize (cpt, AS_FORMAT_STYLE_METAINFO);
-	g_assert_true (as_xml_test_compare_xml (res, xmldata_recommends_requires));
+	g_assert_true (as_xml_test_compare_xml (res, xmldata_relations));
 }
 
 
@@ -1916,6 +1950,72 @@ test_xml_rw_reviews (void)
 	g_assert_true (as_xml_test_compare_xml (res, xmldata_reviews));
 }
 
+/**
+ * test_xml_rw_tags:
+ */
+static void
+test_xml_rw_tags (void)
+{
+	static const gchar *xmldata_tags =
+			"<component>\n"
+			"  <id>org.example.TagsTest</id>\n"
+			"  <tags>\n"
+			"    <tag namespace=\"lvfs\">vendor-2021q1</tag>\n"
+			"    <tag namespace=\"plasma\">featured</tag>\n"
+			"  </tags>\n"
+			"</component>\n";
+	g_autoptr(AsComponent) cpt = NULL;
+	g_autofree gchar *res = NULL;
+
+	/* read */
+	cpt = as_xml_test_read_data (xmldata_tags, AS_FORMAT_STYLE_METAINFO);
+	g_assert_cmpstr (as_component_get_id (cpt), ==, "org.example.TagsTest");
+
+	/* validate */
+	g_assert_true (as_component_has_tag (cpt, "lvfs", "vendor-2021q1"));
+	g_assert_true (as_component_has_tag (cpt, "plasma", "featured"));
+
+	/* write */
+	res = as_xml_test_serialize (cpt, AS_FORMAT_STYLE_METAINFO);
+	g_assert_true (as_xml_test_compare_xml (res, xmldata_tags));
+}
+
+/**
+ * test_xml_rw_branding:
+ */
+static void
+test_xml_rw_branding (void)
+{
+	static const gchar *xmldata_tags =
+			"<component>\n"
+			"  <id>org.example.BrandingTest</id>\n"
+			"  <branding>\n"
+			"    <color type=\"primary\" scheme_preference=\"light\">#ff00ff</color>\n"
+			"    <color type=\"primary\" scheme_preference=\"dark\">#993d3d</color>\n"
+			"  </branding>\n"
+			"</component>\n";
+	g_autoptr(AsComponent) cpt = NULL;
+	g_autofree gchar *res = NULL;
+	AsBranding *branding;
+
+	/* read */
+	cpt = as_xml_test_read_data (xmldata_tags, AS_FORMAT_STYLE_METAINFO);
+	g_assert_cmpstr (as_component_get_id (cpt), ==, "org.example.BrandingTest");
+
+	/* validate */
+	branding = as_component_get_branding (cpt);
+	g_assert_nonnull (branding);
+
+	g_assert_cmpstr (as_branding_get_color (branding, AS_COLOR_KIND_PRIMARY, AS_COLOR_SCHEME_KIND_LIGHT),
+			 ==, "#ff00ff");
+	g_assert_cmpstr (as_branding_get_color (branding, AS_COLOR_KIND_PRIMARY, AS_COLOR_SCHEME_KIND_DARK),
+			 ==, "#993d3d");
+
+	/* write */
+	res = as_xml_test_serialize (cpt, AS_FORMAT_STYLE_METAINFO);
+	g_assert_true (as_xml_test_compare_xml (res, xmldata_tags));
+}
+
 /**
  * main:
  */
@@ -1974,8 +2074,8 @@ main (int argc, char **argv)
 	g_test_add_func ("/XML/Read/Screenshots", test_xml_read_screenshots);
 	g_test_add_func ("/XML/Write/Screenshots", test_xml_write_screenshots);
 
-	g_test_add_func ("/XML/Read/RecommendsRequires", test_xml_read_recommends_requires);
-	g_test_add_func ("/XML/Write/RecommendsRequires", test_xml_write_recommends_requires);
+	g_test_add_func ("/XML/Read/Relations", test_xml_read_relations);
+	g_test_add_func ("/XML/Write/Relations", test_xml_write_relations);
 
 	g_test_add_func ("/XML/Read/Agreements", test_xml_read_agreements);
 	g_test_add_func ("/XML/Write/Agreements", test_xml_write_agreements);
@@ -1985,6 +2085,8 @@ main (int argc, char **argv)
 	g_test_add_func ("/XML/Read/ReleasesLegacy", test_xml_read_releases_legacy);
 
 	g_test_add_func ("/XML/ReadWrite/Reviews", test_xml_rw_reviews);
+	g_test_add_func ("/XML/ReadWrite/Tags", test_xml_rw_tags);
+	g_test_add_func ("/XML/ReadWrite/Branding", test_xml_rw_branding);
 
 	ret = g_test_run ();
 	g_free (datadir);
