@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2012-2021 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2012-2022 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -110,12 +110,12 @@ as_yaml_test_compare_yaml (const gchar *result, const gchar *expected)
 }
 
 /**
- * test_basic:
+ * test_yaml_basic:
  *
  * Test basic functions related to YAML processing.
  */
 static void
-test_basic (void)
+test_yaml_basic (void)
 {
 	g_autoptr(AsMetadata) mdata = NULL;
 	gchar *path;
@@ -216,6 +216,7 @@ test_yamlwrite_misc (void)
 	const gchar *expected_yaml =
 				"Type: firmware\n"
 				"ID: org.example.test.firmware\n"
+				"DateEOL: 2022-02-22T00:00:00Z\n"
 				"Package: fwdummy\n"
 				"Extends:\n"
 				"- org.example.alpha\n"
@@ -307,6 +308,7 @@ test_yamlwrite_misc (void)
 	cpt = as_component_new ();
 	as_component_set_kind (cpt, AS_COMPONENT_KIND_FIRMWARE);
 	as_component_set_id (cpt, "org.example.test.firmware");
+	as_component_set_date_eol (cpt, "2022-02-22");
 	as_component_set_pkgnames (cpt, _PKGNAME1);
 	as_component_set_name (cpt, "Unittest Firmware", "C");
 	as_component_set_name (cpt, "Ünittest Fürmwäre (dummy Eintrag)", "de_DE");
@@ -559,6 +561,7 @@ test_yaml_corrupt_data (void)
 static const gchar *yamldata_simple_fields =
 					"Type: generic\n"
 					"ID: org.example.SimpleTest\n"
+					"DateEOL: 2022-02-22T00:00:00Z\n"
 					"Name:\n"
 					"  C: TestComponent\n"
 					"Summary:\n"
@@ -580,6 +583,7 @@ test_yaml_write_simple (void)
 	cpt = as_component_new ();
 	as_component_set_kind (cpt, AS_COMPONENT_KIND_GENERIC);
 	as_component_set_id (cpt, "org.example.SimpleTest");
+	as_component_set_date_eol (cpt, "2022-02-22");
 
 	as_component_set_name (cpt, "TestComponent", "C");
 	as_component_set_summary (cpt, "Just part of an unittest", "C");
@@ -606,6 +610,8 @@ test_yaml_read_simple (void)
 	g_assert_cmpstr (as_component_get_name (cpt), ==, "TestComponent");
 	g_assert_cmpstr (as_component_get_summary (cpt), ==, "Just part of an unittest");
 	g_assert_cmpstr (as_component_get_name_variant_suffix (cpt), ==, "Generic");
+	g_assert_cmpstr (as_component_get_date_eol (cpt), ==, "2022-02-22T00:00:00Z");
+	g_assert_cmpint (as_component_get_timestamp_eol (cpt), ==, 1645488000);
 }
 
 /**
@@ -1004,28 +1010,31 @@ test_yaml_read_launchable (void)
 	g_assert_cmpstr (g_ptr_array_index (as_launchable_get_entries (launch), 1), ==, "kde4-kool.desktop");
 }
 
-static const gchar *yamldata_requires_recommends_field =
-					"Type: generic\n"
-					"ID: org.example.RelationsTest\n"
-					"Recommends:\n"
-					"- memory: 2500\n"
-					"- modalias: usb:v1130p0202d*\n"
-					"- display_length: <= xlarge\n"
-					"  side: longest\n"
-					"Requires:\n"
-					"- kernel: Linux\n"
-					"  version: '>= 4.15'\n"
-					"- id: org.example.TestDependency\n"
-					"  version: == 1.2\n"
-					"- display_length: 4200\n";
+static const gchar *yamldata_relations_field =
+				"Type: generic\n"
+				"ID: org.example.RelationsTest\n"
+				"Requires:\n"
+				"- kernel: Linux\n"
+				"  version: '>= 4.15'\n"
+				"- id: org.example.TestDependency\n"
+				"  version: == 1.2\n"
+				"- display_length: 4200\n"
+				"Recommends:\n"
+				"- memory: 2500\n"
+				"- modalias: usb:v1130p0202d*\n"
+				"- display_length: <= xlarge\n"
+				"  side: longest\n"
+				"Supports:\n"
+				"- control: gamepad\n"
+				"- control: keyboard\n";
 
 /**
- * test_yaml_write_requires_recommends:
+ * test_yaml_write_relations:
  *
  * Test writing the Requires/Recommends fields.
  */
 static void
-test_yaml_write_requires_recommends (void)
+test_yaml_write_relations (void)
 {
 	g_autoptr(AsComponent) cpt = NULL;
 	g_autofree gchar *res = NULL;
@@ -1035,6 +1044,8 @@ test_yaml_write_requires_recommends (void)
 	g_autoptr(AsRelation) id_relation = NULL;
 	g_autoptr(AsRelation) dl_relation1 = NULL;
 	g_autoptr(AsRelation) dl_relation2 = NULL;
+	g_autoptr(AsRelation) ctl_relation1 = NULL;
+	g_autoptr(AsRelation) ctl_relation2 = NULL;
 
 	cpt = as_component_new ();
 	as_component_set_kind (cpt, AS_COMPONENT_KIND_GENERIC);
@@ -1046,6 +1057,8 @@ test_yaml_write_requires_recommends (void)
 	id_relation = as_relation_new ();
 	dl_relation1 = as_relation_new ();
 	dl_relation2 = as_relation_new ();
+	ctl_relation1 = as_relation_new ();
+	ctl_relation2 = as_relation_new ();
 
 	as_relation_set_kind (mem_relation, AS_RELATION_KIND_RECOMMENDS);
 	as_relation_set_kind (moda_relation, AS_RELATION_KIND_RECOMMENDS);
@@ -1053,6 +1066,8 @@ test_yaml_write_requires_recommends (void)
 	as_relation_set_kind (id_relation, AS_RELATION_KIND_REQUIRES);
 	as_relation_set_kind (dl_relation1, AS_RELATION_KIND_RECOMMENDS);
 	as_relation_set_kind (dl_relation2, AS_RELATION_KIND_REQUIRES);
+	as_relation_set_kind (ctl_relation1, AS_RELATION_KIND_SUPPORTS);
+	as_relation_set_kind (ctl_relation2, AS_RELATION_KIND_SUPPORTS);
 
 	as_relation_set_item_kind (mem_relation, AS_RELATION_ITEM_KIND_MEMORY);
 	as_relation_set_value_int (mem_relation, 2500);
@@ -1078,39 +1093,49 @@ test_yaml_write_requires_recommends (void)
 	as_relation_set_value_int (dl_relation2, 4200);
 	as_relation_set_compare (dl_relation2, AS_RELATION_COMPARE_GE);
 
+	as_relation_set_item_kind (ctl_relation1, AS_RELATION_ITEM_KIND_CONTROL);
+	as_relation_set_item_kind (ctl_relation2, AS_RELATION_ITEM_KIND_CONTROL);
+	as_relation_set_value_control_kind (ctl_relation1, AS_CONTROL_KIND_GAMEPAD);
+	as_relation_set_value_control_kind (ctl_relation2, AS_CONTROL_KIND_KEYBOARD);
+
 	as_component_add_relation (cpt, mem_relation);
 	as_component_add_relation (cpt, moda_relation);
 	as_component_add_relation (cpt, kernel_relation);
 	as_component_add_relation (cpt, id_relation);
 	as_component_add_relation (cpt, dl_relation1);
 	as_component_add_relation (cpt, dl_relation2);
+	as_component_add_relation (cpt, ctl_relation1);
+	as_component_add_relation (cpt, ctl_relation2);
 
 	/* test collection serialization */
 	res = as_yaml_test_serialize (cpt);
-	g_assert_true (as_yaml_test_compare_yaml (res, yamldata_requires_recommends_field));
+	g_assert_true (as_yaml_test_compare_yaml (res, yamldata_relations_field));
 }
 
 /**
- * test_yaml_read_requires_recommends:
+ * test_yaml_read_relations:
  *
  * Test if reading the Requires/Recommends fields works.
  */
 static void
-test_yaml_read_requires_recommends (void)
+test_yaml_read_relations (void)
 {
 	g_autoptr(AsComponent) cpt = NULL;
 	GPtrArray *recommends;
 	GPtrArray *requires;
+	GPtrArray *supports;
 	AsRelation *relation;
 
-	cpt = as_yaml_test_read_data (yamldata_requires_recommends_field, NULL);
+	cpt = as_yaml_test_read_data (yamldata_relations_field, NULL);
 	g_assert_cmpstr (as_component_get_id (cpt), ==, "org.example.RelationsTest");
 
-	recommends = as_component_get_recommends (cpt);
 	requires = as_component_get_requires (cpt);
+	recommends = as_component_get_recommends (cpt);
+	supports = as_component_get_supports (cpt);
 
-	g_assert_cmpint (recommends->len, ==, 3);
 	g_assert_cmpint (requires->len, ==, 3);
+	g_assert_cmpint (recommends->len, ==, 3);
+	g_assert_cmpint (supports->len, ==, 2);
 
 	/* memory relation */
 	relation = AS_RELATION (g_ptr_array_index (recommends, 0));
@@ -1153,6 +1178,16 @@ test_yaml_read_requires_recommends (void)
 	g_assert_cmpint (as_relation_get_item_kind (relation), ==, AS_RELATION_ITEM_KIND_DISPLAY_LENGTH);
 	g_assert_cmpint (as_relation_get_value_px (relation), ==, 4200);
 	g_assert_cmpint (as_relation_get_compare (relation), ==, AS_RELATION_COMPARE_GE);
+
+	/* control relation */
+	relation = AS_RELATION (g_ptr_array_index (supports, 0));
+	g_assert_cmpint (as_relation_get_kind (relation), ==, AS_RELATION_KIND_SUPPORTS);
+	g_assert_cmpint (as_relation_get_item_kind (relation), ==, AS_RELATION_ITEM_KIND_CONTROL);
+	g_assert_cmpint (as_relation_get_value_control_kind (relation), ==, AS_CONTROL_KIND_GAMEPAD);
+	relation = AS_RELATION (g_ptr_array_index (supports, 1));
+	g_assert_cmpint (as_relation_get_kind (relation), ==, AS_RELATION_KIND_SUPPORTS);
+	g_assert_cmpint (as_relation_get_item_kind (relation), ==, AS_RELATION_ITEM_KIND_CONTROL);
+	g_assert_cmpint (as_relation_get_value_control_kind (relation), ==, AS_CONTROL_KIND_KEYBOARD);
 }
 
 
@@ -1161,7 +1196,7 @@ static const gchar *yamldata_agreements =
 				"ID: org.example.AgreementsTest\n"
 				"Agreements:\n"
 				"- type: eula\n"
-				"  version_id: 1.2.3a\n"
+				"  version-id: 1.2.3a\n"
 				"  sections:\n"
 				"  - type: intro\n"
 				"    name:\n"
@@ -1623,6 +1658,75 @@ test_yaml_read_releases (void)
 	g_assert_cmpstr (as_artifact_get_platform (af), ==, "x86_64-linux-gnu");
 }
 
+/**
+ * test_yaml_rw_tags:
+ */
+static void
+test_yaml_rw_tags (void)
+{
+	static const gchar *yamldata_tags =
+			"Type: generic\n"
+			"ID: org.example.TagsTest\n"
+			"Tags:\n"
+			"- namespace: lvfs\n"
+			"  tag: vendor-2021q1\n"
+			"- namespace: plasma\n"
+			"  tag: featured\n";
+	g_autoptr(AsComponent) cpt = NULL;
+	g_autofree gchar *res = NULL;
+
+	/* read */
+	cpt = as_yaml_test_read_data (yamldata_tags, NULL);
+	g_assert_cmpstr (as_component_get_id (cpt), ==, "org.example.TagsTest");
+
+	/* validate */
+	g_assert_true (as_component_has_tag (cpt, "lvfs", "vendor-2021q1"));
+	g_assert_true (as_component_has_tag (cpt, "plasma", "featured"));
+
+	/* write */
+	res = as_yaml_test_serialize (cpt);
+	g_assert_true (as_yaml_test_compare_yaml (res, yamldata_tags));
+}
+
+/**
+ * test_yaml_rw_branding:
+ */
+static void
+test_yaml_rw_branding (void)
+{
+	static const gchar *yamldata_tags =
+			"Type: generic\n"
+			"ID: org.example.BrandingTest\n"
+			"Branding:\n"
+			"  colors:\n"
+			"  - type: primary\n"
+			"    scheme-preference: light\n"
+			"    value: '#ff00ff'\n"
+			"  - type: primary\n"
+			"    scheme-preference: dark\n"
+			"    value: '#993d3d'\n";
+	g_autoptr(AsComponent) cpt = NULL;
+	g_autofree gchar *res = NULL;
+	AsBranding *branding;
+
+	/* read */
+	cpt = as_yaml_test_read_data (yamldata_tags, NULL);
+	g_assert_cmpstr (as_component_get_id (cpt), ==, "org.example.BrandingTest");
+
+	/* validate */
+	branding = as_component_get_branding (cpt);
+	g_assert_nonnull (branding);
+
+	g_assert_cmpstr (as_branding_get_color (branding, AS_COLOR_KIND_PRIMARY, AS_COLOR_SCHEME_KIND_LIGHT),
+			 ==, "#ff00ff");
+	g_assert_cmpstr (as_branding_get_color (branding, AS_COLOR_KIND_PRIMARY, AS_COLOR_SCHEME_KIND_DARK),
+			 ==, "#993d3d");
+
+	/* write */
+	res = as_yaml_test_serialize (cpt);
+	g_assert_true (as_yaml_test_compare_yaml (res, yamldata_tags));
+}
+
 /**
  * main:
  */
@@ -1647,7 +1751,7 @@ main (int argc, char **argv)
 	/* only critical and error are fatal */
 	g_log_set_fatal_mask (NULL, G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL);
 
-	g_test_add_func ("/YAML/Basic", test_basic);
+	g_test_add_func ("/YAML/Basic", test_yaml_basic);
 	g_test_add_func ("/YAML/Write/Misc", test_yamlwrite_misc);
 
 	g_test_add_func ("/YAML/Read/CorruptData", test_yaml_corrupt_data);
@@ -1673,8 +1777,8 @@ main (int argc, char **argv)
 	g_test_add_func ("/YAML/Read/Launchable", test_yaml_read_launchable);
 	g_test_add_func ("/YAML/Write/Launchable", test_yaml_write_launchable);
 
-	g_test_add_func ("/YAML/Read/RequiresRecommends", test_yaml_read_requires_recommends);
-	g_test_add_func ("/YAML/Write/RequiresRecommends", test_yaml_write_requires_recommends);
+	g_test_add_func ("/YAML/Read/Relations", test_yaml_read_relations);
+	g_test_add_func ("/YAML/Write/Relations", test_yaml_write_relations);
 
 	g_test_add_func ("/YAML/Read/Agreements", test_yaml_read_agreements);
 	g_test_add_func ("/YAML/Write/Agreements", test_yaml_write_agreements);
@@ -1685,6 +1789,9 @@ main (int argc, char **argv)
 	g_test_add_func ("/YAML/Read/Releases", test_yaml_read_releases);
 	g_test_add_func ("/YAML/Write/Releases", test_yaml_write_releases);
 
+	g_test_add_func ("/YAML/ReadWrite/Tags", test_yaml_rw_tags);
+	g_test_add_func ("/YAML/ReadWrite/Branding", test_yaml_rw_branding);
+
 	ret = g_test_run ();
 	g_free (datadir);
 	return ret;
