@@ -1,20 +1,20 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2012-2021 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2012-2022 Matthias Klumpp <matthias@tenstral.net>
  *
- * Licensed under the GNU General Public License Version 2
+ * Licensed under the GNU Lesser General Public License Version 2.1
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 2 of the license, or
+ * it under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation, either version 2.1 of the license, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -26,6 +26,7 @@
 #include <stdio.h>
 
 #include "as-profile.h"
+#include "as-utils-private.h"
 
 #include "ascli-utils.h"
 #include "ascli-actions-mdata.h"
@@ -66,7 +67,7 @@ const GOptionEntry data_collection_options[] = {
 		G_OPTION_ARG_NONE,
 		&optn_no_cache,
 		/* TRANSLATORS: ascli flag description for: --no-cache */
-		N_("Make request without any caching."),
+		N_("Ignore cache age and build a fresh cache before performing the query."),
 		NULL },
 	{ NULL }
 };
@@ -171,7 +172,8 @@ as_client_new_subcommand_option_context (const gchar *command, const GOptionEntr
 
 	opt_context = g_option_context_new ("- AppStream CLI.");
 	g_option_context_set_help_enabled (opt_context, TRUE);
-	g_option_context_add_main_entries (opt_context, entries, NULL);
+	if (entries != NULL)
+		g_option_context_add_main_entries (opt_context, entries, NULL);
 
 	/* set the summary text */
 	summary = as_client_get_summary_for (command);
@@ -235,7 +237,7 @@ as_client_run_refresh_cache (const gchar *command, char **argv, int argc)
 	g_autoptr(GOptionContext) opt_context = NULL;
 	gint ret;
 	gboolean optn_force = FALSE;
-	gboolean optn_user = FALSE;
+	g_auto(GStrv) optn_sources = NULL;
 
 	const GOptionEntry refresh_options[] = {
 		{ "force", (gchar) 0, 0,
@@ -244,11 +246,11 @@ as_client_run_refresh_cache (const gchar *command, char **argv, int argc)
 			/* TRANSLATORS: ascli flag description for: --force */
 			_("Enforce a cache refresh."),
 			NULL },
-		{ "user", (gchar) 0, 0,
-			G_OPTION_ARG_NONE,
-			&optn_user,
-			/* TRANSLATORS: ascli flag description for: --user */
-			_("Update the user-specific instead of the system-wide cache."),
+		{ "source", (gchar) 0, 0,
+			G_OPTION_ARG_STRING_ARRAY,
+			&optn_sources,
+			/* TRANSLATORS: ascli flag description for: --source in a refresh action. Don't translate strings in backticks: `name` */
+			_("Limit cache refresh to data from a specific source, e.g. `os` or `flatpak`. May be specified multiple times."),
 			NULL },
 		{ NULL }
 	};
@@ -263,7 +265,7 @@ as_client_run_refresh_cache (const gchar *command, char **argv, int argc)
 
 	return ascli_refresh_cache (optn_cachepath,
 				    optn_datapath,
-				    optn_user,
+				    (const gchar * const*) optn_sources,
 				    optn_force);
 }
 
@@ -455,6 +457,30 @@ as_client_run_validate_tree (const gchar *command, char **argv, int argc)
 	}
 }
 
+/**
+ * as_client_run_check_license:
+ *
+ * Print license information.
+ */
+static int
+as_client_run_check_license (const gchar *command, char **argv, int argc)
+{
+	g_autoptr(GOptionContext) opt_context = NULL;
+	gint ret;
+
+	opt_context = as_client_new_subcommand_option_context (command, NULL);
+	ret = as_client_option_context_parse (opt_context, command, &argc, &argv);
+	if (ret != 0)
+		return ret;
+
+	if (argc != 3) {
+		/* TRANSLATORS: ascli check-license is missing its parameter */
+		ascli_print_stderr (_("No license, license expression or license exception string was provided."));
+		return 4;
+	}
+	return ascli_check_license (argv[2]);
+}
+
 /**
  * as_client_run_put:
  *
@@ -499,6 +525,24 @@ as_client_run_put (const gchar *command, char **argv, int argc)
 	return ascli_put_metainfo (fname, optn_origin, optn_usermode);
 }
 
+static const gchar *optn_bundle_type = NULL;
+static gboolean optn_choose_first = FALSE;
+
+const GOptionEntry pkgmanage_options[] = {
+	{ "bundle-type", 0, 0,
+		G_OPTION_ARG_STRING,
+		&optn_bundle_type,
+		/* TRANSLATORS: ascli flag description for: --bundle-type (part of the "remove" and "install" subcommands) */
+		N_("Limit the command to use only components from the given bundling system (`flatpak` or `package`)."), NULL },
+	{ "first", 0, 0,
+		G_OPTION_ARG_NONE,
+		&optn_choose_first,
+		/* TRANSLATORS: ascli flag description for: --first (part of the "remove" and "install" subcommands) */
+		N_("Do not ask for which software component should be used and always choose the first entry."),
+		NULL },
+	{ NULL }
+};
+
 /**
  * as_client_run_install:
  *
@@ -507,7 +551,15 @@ as_client_run_put (const gchar *command, char **argv, int argc)
 static int
 as_client_run_install (const gchar *command, char **argv, int argc)
 {
+	g_autoptr(GOptionContext) opt_context = NULL;
 	const gchar *value = NULL;
+	AsBundleKind bundle_kind;
+	gint ret;
+
+	opt_context = as_client_new_subcommand_option_context (command, pkgmanage_options);
+	ret = as_client_option_context_parse (opt_context, command, &argc, &argv);
+	if (ret != 0)
+		return ret;
 
 	if (argc > 2)
 		value = argv[2];
@@ -516,7 +568,16 @@ as_client_run_install (const gchar *command, char **argv, int argc)
 		return 1;
 	}
 
-	return ascli_install_component (value);
+	bundle_kind = as_bundle_kind_from_string (optn_bundle_type);
+	if (optn_bundle_type != NULL && bundle_kind == AS_BUNDLE_KIND_UNKNOWN) {
+		/* TRANSLATORS: ascli install currently only supports two values for --bundle-type. */
+		ascli_print_stderr (_("No valid bundle kind was specified. Only `package` and `flatpak` are currently recognized."));
+		return ASCLI_EXIT_CODE_BAD_INPUT;
+	}
+
+	return ascli_install_component (value,
+					bundle_kind,
+					optn_choose_first);
 }
 
 /**
@@ -527,7 +588,15 @@ as_client_run_install (const gchar *command, char **argv, int argc)
 static int
 as_client_run_remove (const gchar *command, char **argv, int argc)
 {
+	g_autoptr(GOptionContext) opt_context = NULL;
 	const gchar *value = NULL;
+	AsBundleKind bundle_kind;
+	gint ret;
+
+	opt_context = as_client_new_subcommand_option_context (command, pkgmanage_options);
+	ret = as_client_option_context_parse (opt_context, command, &argc, &argv);
+	if (ret != 0)
+		return ret;
 
 	if (argc > 2)
 		value = argv[2];
@@ -536,7 +605,16 @@ as_client_run_remove (const gchar *command, char **argv, int argc)
 		return 1;
 	}
 
-	return ascli_remove_component (value);
+	bundle_kind = as_bundle_kind_from_string (optn_bundle_type);
+	if (optn_bundle_type != NULL && bundle_kind == AS_BUNDLE_KIND_UNKNOWN) {
+		/* TRANSLATORS: ascli install currently only supports two values for --bundle-type. */
+		ascli_print_stderr (_("No valid bundle kind was specified. Only `package` and `flatpak` are currently recognized."));
+		return ASCLI_EXIT_CODE_BAD_INPUT;
+	}
+
+	return ascli_remove_component (value,
+					bundle_kind,
+					optn_choose_first);
 }
 
 /**
@@ -801,7 +879,8 @@ as_client_run_news_to_metainfo (const gchar *command, char **argv, int argc)
 {
 	g_autoptr(GOptionContext) opt_context = NULL;
 	const gchar *optn_format_text = NULL;
-	guint optn_limit = 0;
+	gint optn_limit = 0;
+	gint optn_translatable_n = -1;
 	const gchar *mi_fname = NULL;
 	const gchar *news_fname = NULL;
 	const gchar *out_fname = NULL;
@@ -812,12 +891,20 @@ as_client_run_news_to_metainfo (const gchar *command, char **argv, int argc)
 			G_OPTION_ARG_STRING,
 			&optn_format_text,
 			/* TRANSLATORS: ascli flag description for: --format as part of the news-to-metainfo command */
-			N_("Assume the input file is in the selected format ('yaml' or 'text')."), NULL },
-		{ "limit", 0, 0,
+			N_("Assume the input file is in the selected format ('yaml' or 'text')."),
+			NULL },
+		{ "limit", 'l', 0,
 			G_OPTION_ARG_INT,
 			&optn_limit,
 			/* TRANSLATORS: ascli flag description for: --limit as part of the news-to-metainfo command */
-			N_("Limit the number of release entries that end up in the metainfo file (0 for unlimited)."), NULL },
+			N_("Limit the number of release entries that end up in the metainfo file (<= 0 for unlimited)."),
+			NULL },
+		{ "translatable-count", 't', 0,
+			G_OPTION_ARG_INT,
+			&optn_translatable_n,
+			/* TRANSLATORS: ascli flag description for: --translatable-count as part of the news-to-metainfo command */
+			N_("Set the number of releases that should have descriptions marked for translation (latest releases are translated first, -1 for unlimited)."),
+			NULL },
 		{ NULL }
 	};
 
@@ -837,6 +924,7 @@ as_client_run_news_to_metainfo (const gchar *command, char **argv, int argc)
 					mi_fname,
 					out_fname,
 					optn_limit,
+					optn_translatable_n,
 					optn_format_text);
 }
 
@@ -878,6 +966,15 @@ as_client_run_metainfo_to_news (const gchar *command, char **argv, int argc)
 					optn_format_text);
 }
 
+/**
+ * as_client_check_compose_available:
+ */
+static gboolean
+as_client_check_compose_available (void)
+{
+	return g_file_test (LIBEXECDIR "/appstreamcli-compose", G_FILE_TEST_EXISTS);
+}
+
 /**
  * as_client_run_compose:
  *
@@ -891,14 +988,22 @@ as_client_run_compose (const gchar *command, char **argv, int argc)
 	g_autofree const gchar **asc_argv = NULL;
 
 	if (!g_file_test (ascompose_exe, G_FILE_TEST_EXISTS)) {
+		/* TRANSLATORS: appstreamcli-compose was not found */
 		ascli_print_stderr (_("Compose binary '%s' was not found! Can not continue."), ascompose_exe);
+		ascli_print_stderr (_("You may be able to install the AppStream Compose addon via: `%s`"),
+				    "sudo appstreamcli install org.freedesktop.appstream.compose");
 		return 4;
 	}
 
 	asc_argv = g_new0 (const gchar*, argc + 2);
 	asc_argv[0] = ascompose_exe;
-	for (gint i = 0; i < argc; i++)
-		asc_argv[i+1] = argv[i];
+	if (argc < 2) {
+		/* TRANSLATORS: Unexpected number of parameters on the command-line */
+		ascli_print_stderr (_("Invalid number of parameters"));
+		return 5;
+	}
+	for (gint i = 2; i < argc; i++)
+		asc_argv[i-1] = argv[i];
 
 	return execv(ascompose_exe, (gchar * const*) asc_argv);
 }
@@ -974,6 +1079,7 @@ static gchar*
 as_client_get_help_summary (GPtrArray *commands)
 {
 	guint current_block_id = 0;
+	gboolean compose_available = FALSE;
 	g_autoptr(GArray) blocks_maxlen = NULL;
 	GString *string = g_string_new ("");
 
@@ -982,6 +1088,7 @@ as_client_get_help_summary (GPtrArray *commands)
 				/* these are commands we can use with appstreamcli */
 				_("Subcommands:"));
 
+	compose_available = as_client_check_compose_available ();
 	blocks_maxlen = g_array_new (FALSE, FALSE, sizeof (guint));
 	for (guint i = 0; i < commands->len; i++) {
 		guint nlen;
@@ -1005,6 +1112,10 @@ as_client_get_help_summary (GPtrArray *commands)
 		g_autofree gchar *summary_wrap = NULL;
 		AsCliCommandItem *item = (AsCliCommandItem *) g_ptr_array_index (commands, i);
 
+		/* don't display compose help if ascompose binary was not found */
+		if (!compose_available && g_strcmp0 (item->name, "compose") == 0)
+			continue;
+
 		if (item->block_id != current_block_id) {
 			current_block_id = item->block_id;
 			g_string_append (string, "\n");
@@ -1133,6 +1244,11 @@ as_client_run (char **argv, int argc)
 			/* TRANSLATORS: `appstreamcli validate-tree` command description. */
 			_("Validate an installed file-tree of an application for valid metadata."),
 			as_client_run_validate_tree);
+	ascli_add_cmd (commands,
+			2, "check-license", NULL, "LICENSE",
+			/* TRANSLATORS: `appstreamcli `check-license` command description. */
+			_("Check license string for validity and print details about it."),
+			as_client_run_check_license);
 
 	ascli_add_cmd (commands,
 			3, "install", NULL, "COMPONENT-ID",
@@ -1166,7 +1282,7 @@ as_client_run (char **argv, int argc)
 			_("Convert collection XML to YAML or vice versa."),
 			as_client_run_convert);
 	ascli_add_cmd (commands,
-			4, "compare-versions", "vercmp", "VER1 [COMP] VER2",
+			4, "vercmp", "compare-versions", "VER1 [COMP] VER2",
 			/* TRANSLATORS: `appstreamcli vercmp` command description. */
 			_("Compare two version numbers."),
 			as_client_run_compare_versions);
@@ -1191,12 +1307,11 @@ as_client_run (char **argv, int argc)
 			/* TRANSLATORS: `appstreamcli metainfo-to-news` command description. */
 			_("Write NEWS text or YAML file with information from a metainfo file."),
 			as_client_run_metainfo_to_news);
-	if (g_file_test (LIBEXECDIR "/appstreamcli-compose", G_FILE_TEST_EXISTS))
-		ascli_add_cmd (commands,
-				5, "compose", NULL, NULL,
-				/* TRANSLATORS: `appstreamcli compose` command description. */
-				_("Compose AppStream collection metadata from directory trees."),
-				as_client_run_compose);
+	ascli_add_cmd (commands,
+			5, "compose", NULL, NULL,
+			/* TRANSLATORS: `appstreamcli compose` command description. */
+			_("Compose AppStream collection metadata from directory trees."),
+			as_client_run_compose);
 
 	/* we handle the unknown options later in the individual subcommands */
 	g_option_context_set_ignore_unknown_options (opt_context, TRUE);
@@ -1246,13 +1361,22 @@ as_client_run (char **argv, int argc)
 		optn_nonet = TRUE;
 	}
 
+	/* set some global defaults, in case we run as root in an unsafe environment */
+	if (as_utils_is_root ()) {
+		/* users umask shouldn't interfere with us creating new files when we are root */
+		as_reset_umask ();
+
+		/* ensure we never start gvfsd as root: https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=852696 */
+		g_setenv ("GIO_USE_VFS", "local", TRUE);
+	}
+
 	ascli_set_output_colored (!optn_no_color);
 
 	/* if out terminal is no tty, disable colors automatically */
 	if (!isatty (fileno (stdout)))
 		ascli_set_output_colored (FALSE);
 
-	/* don't let gvfsd start it's own session bus: https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=852696 */
+	/* don't let gvfsd start its own session bus: https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=852696 */
 	g_setenv ("GIO_USE_VFS", "local", TRUE);
 
 	/* prepare profiler */
